# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2021-2022 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""This module contains helper classes/functions for fixtures."""
import logging
from typing import Any, Dict, List, Tuple, cast

import docker
import pytest
from eth_account import Account

from tests.conftest import GANACHE_CONFIGURATION
from tests.helpers.constants import KEY_PAIRS, LOCALHOST
from tests.helpers.docker.amm_net import AMMNetDockerImage
from tests.helpers.docker.base import DockerBaseTest, DockerImage
from tests.helpers.docker.ganache import (
    DEFAULT_GANACHE_ADDR,
    DEFAULT_GANACHE_PORT,
    GanacheDockerImage,
)
from tests.helpers.docker.gnosis_safe_net import (
    DEFAULT_HARDHAT_ADDR,
    DEFAULT_HARDHAT_PORT,
    GnosisSafeNetDockerImage,
)
from tests.helpers.docker.tendermint import (
    FlaskTendermintDockerImage,
    TendermintDockerImage,
)


logger = logging.getLogger(__name__)


@pytest.mark.integration
class UseTendermint:
    """Inherit from this class to use Tendermint."""

    _tendermint_image: TendermintDockerImage
    tendermint_port: int

    @pytest.fixture(autouse=True, scope="class")
    def _start_tendermint(
        self, tendermint: TendermintDockerImage, tendermint_port: Any
    ) -> None:
        """Start a Tendermint image."""
        cls = type(self)
        cls._tendermint_image = tendermint
        cls.tendermint_port = tendermint_port

    @property
    def abci_host(self) -> str:
        """Get the abci host address."""
        return self._tendermint_image.abci_host

    @property
    def abci_port(self) -> int:
        """Get the abci port."""
        return self._tendermint_image.abci_port

    @property
    def node_address(self) -> str:
        """Get the node address."""
        return f"http://{LOCALHOST}:{self.tendermint_port}"


@pytest.mark.integration
class UseFlaskTendermintNode:
    """Inherit from this class to use flask server with Tendermint."""

    @pytest.fixture(autouse=True)
    def _start_tendermint(
        self, flask_tendermint: FlaskTendermintDockerImage, tendermint_port: Any
    ) -> None:
        """Start a Tendermint image."""
        self._tendermint_image = flask_tendermint
        self.tendermint_port = tendermint_port

    @property
    def p2p_seeds(self) -> List[str]:
        """Get the p2p seeds."""
        return self._tendermint_image.p2p_seeds

    def get_node_name(self, i: int) -> str:
        """Get the node's name."""
        return self._tendermint_image.get_node_name(i)

    def get_abci_port(self, i: int) -> int:
        """Get the ith rpc port."""
        return self._tendermint_image.get_abci_port(i)

    def get_port(self, i: int) -> int:
        """Get the ith port."""
        return self._tendermint_image.get_port(i)

    def get_com_port(self, i: int) -> int:
        """Get the ith com port."""
        return self._tendermint_image.get_com_port(i)

    def get_laddr(self, i: int, p2p: bool = False) -> str:
        """Get the ith rpc port."""
        return self._tendermint_image.get_addr("tcp://", i, p2p)

    def health_check(self, **kwargs: Any) -> None:
        """Perform a health check."""
        self._tendermint_image.health_check(**kwargs)


@pytest.mark.integration
class UseGnosisSafeHardHatNet:
    """Inherit from this class to use HardHat local net with Gnosis-Safe deployed."""

    key_pairs: List[Tuple[str, str]] = KEY_PAIRS

    @classmethod
    @pytest.fixture(autouse=True)
    def _start_hardhat(
        cls, gnosis_safe_hardhat_scope_function: Any, hardhat_port: Any, key_pairs: Any
    ) -> None:
        """Start an HardHat instance."""
        cls.key_pairs = key_pairs


@pytest.mark.integration
class UseGanache:
    """Inherit from this class to use Ganache."""

    key_pairs: List[Tuple[str, str]] = []

    @classmethod
    @pytest.fixture(autouse=True)
    def _start_ganache(cls, ganache: Any, ganache_configuration: Any) -> None:
        """Start Ganache instance."""
        cls.key_pairs = cast(
            List[Tuple[str, str]],
            [
                key if type(key) == tuple else (Account.from_key(key).address, key)
                for key, _ in ganache_configuration.get("accounts_balances", [])
            ],
        )


class GanacheBaseTest(DockerBaseTest):
    """Base pytest class for Ganache."""

    addr: str = DEFAULT_GANACHE_ADDR
    port: int = DEFAULT_GANACHE_PORT
    configuration: Dict = GANACHE_CONFIGURATION

    @classmethod
    def setup_class_kwargs(cls) -> Dict[str, Any]:
        """Get kwargs for _setup_class call."""
        setup_class_kwargs = {
            "key_pairs": cls.key_pairs(),
            "url": cls.url(),
        }
        return setup_class_kwargs

    @classmethod
    def _build_image(cls) -> DockerImage:
        """Build the image."""
        client = docker.from_env()
        return GanacheDockerImage(client, cls.addr, cls.port, config=cls.configuration)

    @classmethod
    def key_pairs(cls) -> List[Tuple[str, str]]:
        """Get the key pairs which are funded."""
        key_pairs = cast(
            List[Tuple[str, str]],
            [
                key if type(key) == tuple else (Account.from_key(key).address, key)
                for key, _ in cls.configuration.get("accounts_balances", [])
            ],
        )
        return key_pairs

    @classmethod
    def url(cls) -> str:
        """Get the url under which the image is reachable."""
        return f"{cls.addr}:{cls.port}"


class HardHatBaseTest(DockerBaseTest):
    """Base pytest class for HardHat."""

    addr: str = DEFAULT_HARDHAT_ADDR
    port: int = DEFAULT_HARDHAT_PORT

    @classmethod
    def setup_class_kwargs(cls) -> Dict[str, Any]:
        """Get kwargs for _setup_class call."""
        setup_class_kwargs = {
            "key_pairs": cls.key_pairs(),
            "url": cls.url(),
        }
        return setup_class_kwargs

    @classmethod
    def key_pairs(cls) -> List[Tuple[str, str]]:
        """Get the key pairs which are funded."""
        return KEY_PAIRS

    @classmethod
    def url(cls) -> str:
        """Get the url under which the image is reachable."""
        return f"{cls.addr}:{cls.port}"


class HardHatGnosisBaseTest(HardHatBaseTest):
    """Base pytest class for HardHat with Gnosis deployed."""

    @classmethod
    def _build_image(cls) -> DockerImage:
        """Build the image."""
        client = docker.from_env()
        return GnosisSafeNetDockerImage(client, cls.addr, cls.port)


class HardHatAMMBaseTest(HardHatBaseTest):
    """Base pytest class for HardHat with Gnosis and Uniswap deployed."""

    @classmethod
    def _build_image(cls) -> DockerImage:
        """Build the image."""
        client = docker.from_env()
        return AMMNetDockerImage(client, cls.addr, cls.port)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2021-2022 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Tests for valory/hello_world_abci skill's behaviours."""
import json
import time
from copy import copy
from pathlib import Path
from tempfile import TemporaryDirectory
from typing import Any, Dict, Type, cast
from unittest import mock

from aea.helpers.transaction.base import SignedMessage
from aea.skills.base import SkillContext
from aea.test_tools.test_skill import BaseSkillTestCase

from packages.open_aea.protocols.signing import SigningMessage
from packages.valory.connections.http_client.connection import (
    PUBLIC_ID as HTTP_CLIENT_PUBLIC_ID,
)
from packages.valory.protocols.http import HttpMessage
from packages.valory.skills.abstract_round_abci.base import (
    AbciAppDB,
    AbstractRound,
    BaseSynchronizedData,
    BaseTxPayload,
    OK_CODE,
    _MetaPayload,
)
from packages.valory.skills.abstract_round_abci.behaviour_utils import BaseBehaviour
from packages.valory.skills.abstract_round_abci.behaviours import AbstractRoundBehaviour
from packages.valory.skills.hello_world_abci.behaviours import (
    HelloWorldRoundBehaviour,
    PrintMessageBehaviour,
    RegistrationBehaviour,
    ResetAndPauseBehaviour,
    SelectKeeperBehaviour,
)
from packages.valory.skills.hello_world_abci.handlers import HttpHandler, SigningHandler
from packages.valory.skills.hello_world_abci.rounds import Event, SynchronizedData

from tests.conftest import ROOT_DIR


class HelloWorldAbciFSMBehaviourBaseCase(BaseSkillTestCase):
    """Base case for testing PriceEstimation FSMBehaviour."""

    path_to_skill = Path(ROOT_DIR, "packages", "valory", "skills", "hello_world_abci")

    hello_world_abci_behaviour: HelloWorldRoundBehaviour
    http_handler: HttpHandler
    signing_handler: SigningHandler
    old_tx_type_to_payload_cls: Dict[str, Type[BaseTxPayload]]
    synchronized_data: SynchronizedData
    benchmark_dir: TemporaryDirectory

    @classmethod
    def setup(cls, **kwargs: Any) -> None:
        """Setup the test class."""
        # we need to store the current value of the meta-class attribute
        # _MetaPayload.transaction_type_to_payload_cls, and restore it
        # in the teardown function. We do a shallow copy so we avoid
        # to modify the old mapping during the execution of the tests.
        cls.old_tx_type_to_payload_cls = copy(
            _MetaPayload.transaction_type_to_payload_cls
        )
        _MetaPayload.transaction_type_to_payload_cls = {}
        super().setup()
        assert cls._skill.skill_context._agent_context is not None
        cls._skill.skill_context._agent_context.identity._default_address_key = (
            "ethereum"
        )
        cls._skill.skill_context._agent_context._default_ledger_id = "ethereum"
        cls.hello_world_abci_behaviour = cast(
            HelloWorldRoundBehaviour,
            cls._skill.skill_context.behaviours.main,
        )
        cls.http_handler = cast(HttpHandler, cls._skill.skill_context.handlers.http)
        cls.signing_handler = cast(
            SigningHandler, cls._skill.skill_context.handlers.signing
        )

        cls.hello_world_abci_behaviour.setup()
        cls._skill.skill_context.state.setup()
        cls._skill.skill_context.state.round_sequence.end_sync()

        cls.benchmark_dir = TemporaryDirectory()
        cls._skill.skill_context.benchmark_tool.log_dir = Path(cls.benchmark_dir.name)

        assert (
            cast(
                BaseBehaviour, cls.hello_world_abci_behaviour.current_behaviour
            ).behaviour_id
            == cls.hello_world_abci_behaviour.initial_behaviour_cls.behaviour_id
        )
        cls.synchronized_data = SynchronizedData(
            AbciAppDB(
                initial_data=dict(
                    most_voted_keeper_address=["most_voted_keeper_address"],
                ),
            )
        )

    def fast_forward_to_behaviour(
        self,
        behaviour: AbstractRoundBehaviour,
        behaviour_id: str,
        synchronized_data: BaseSynchronizedData,
    ) -> None:
        """Fast forward the FSM to a behaviour."""
        next_behaviour = {s.behaviour_id: s for s in behaviour.behaviours}[behaviour_id]
        assert next_behaviour is not None, f"Behaviour {behaviour_id} not found"
        next_behaviour = cast(Type[BaseBehaviour], next_behaviour)
        behaviour.current_behaviour = next_behaviour(
            name=next_behaviour.behaviour_id, skill_context=behaviour.context
        )
        self.skill.skill_context.state.round_sequence.abci_app._round_results.append(
            synchronized_data
        )
        self.skill.skill_context.state.round_sequence.abci_app._extend_previous_rounds_with_current_round()
        self.skill.skill_context.behaviours.main._last_round_height = (
            self.skill.skill_context.state.round_sequence.abci_app.current_round_height
        )
        self.skill.skill_context.state.round_sequence.abci_app._current_round = (
            next_behaviour.matching_round(
                synchronized_data, self.skill.skill_context.params.consensus_params
            )
        )

    def mock_http_request(self, request_kwargs: Dict, response_kwargs: Dict) -> None:
        """
        Mock http request.

        :param request_kwargs: keyword arguments for request check.
        :param response_kwargs: keyword arguments for mock response.
        """

        self.assert_quantity_in_outbox(1)
        actual_http_message = self.get_message_from_outbox()
        assert actual_http_message is not None, "No message in outbox."
        has_attributes, error_str = self.message_has_attributes(
            actual_message=actual_http_message,
            message_type=HttpMessage,
            performative=HttpMessage.Performative.REQUEST,
            to=str(HTTP_CLIENT_PUBLIC_ID),
            sender=str(self.skill.skill_context.skill_id),
            **request_kwargs,
        )
        assert has_attributes, error_str
        self.hello_world_abci_behaviour.act_wrapper()
        self.assert_quantity_in_outbox(0)
        incoming_message = self.build_incoming_message(
            message_type=HttpMessage,
            dialogue_reference=(actual_http_message.dialogue_reference[0], "stub"),
            performative=HttpMessage.Performative.RESPONSE,
            target=actual_http_message.message_id,
            message_id=-1,
            to=str(self.skill.skill_context.skill_id),
            sender=str(HTTP_CLIENT_PUBLIC_ID),
            **response_kwargs,
        )
        self.http_handler.handle(incoming_message)
        self.hello_world_abci_behaviour.act_wrapper()

    def mock_signing_request(self, request_kwargs: Dict, response_kwargs: Dict) -> None:
        """Mock signing request."""
        self.assert_quantity_in_decision_making_queue(1)
        actual_signing_message = self.get_message_from_decision_maker_inbox()
        assert actual_signing_message is not None, "No message in outbox."
        has_attributes, error_str = self.message_has_attributes(
            actual_message=actual_signing_message,
            message_type=SigningMessage,
            to="dummy_decision_maker_address",
            sender=str(self.skill.skill_context.skill_id),
            **request_kwargs,
        )
        assert has_attributes, error_str
        incoming_message = self.build_incoming_message(
            message_type=SigningMessage,
            dialogue_reference=(actual_signing_message.dialogue_reference[0], "stub"),
            target=actual_signing_message.message_id,
            message_id=-1,
            to=str(self.skill.skill_context.skill_id),
            sender="dummy_decision_maker_address",
            **response_kwargs,
        )
        self.signing_handler.handle(incoming_message)
        self.hello_world_abci_behaviour.act_wrapper()

    def mock_a2a_transaction(
        self,
    ) -> None:
        """Performs mock a2a transaction."""

        self.mock_signing_request(
            request_kwargs=dict(
                performative=SigningMessage.Performative.SIGN_MESSAGE,
            ),
            response_kwargs=dict(
                performative=SigningMessage.Performative.SIGNED_MESSAGE,
                signed_message=SignedMessage(
                    ledger_id="ethereum", body="stub_signature"
                ),
            ),
        )

        self.mock_http_request(
            request_kwargs=dict(
                method="GET",
                headers="",
                version="",
                body=b"",
            ),
            response_kwargs=dict(
                version="",
                status_code=200,
                status_text="",
                headers="",
                body=json.dumps({"result": {"hash": "", "code": OK_CODE}}).encode(
                    "utf-8"
                ),
            ),
        )
        self.mock_http_request(
            request_kwargs=dict(
                method="GET",
                headers="",
                version="",
                body=b"",
            ),
            response_kwargs=dict(
                version="",
                status_code=200,
                status_text="",
                headers="",
                body=json.dumps({"result": {"tx_result": {"code": OK_CODE}}}).encode(
                    "utf-8"
                ),
            ),
        )

    def end_round(
        self,
    ) -> None:
        """Ends round early to cover `wait_for_end` generator."""
        current_behaviour = cast(
            BaseBehaviour, self.hello_world_abci_behaviour.current_behaviour
        )
        if current_behaviour is None:
            return
        current_behaviour = cast(BaseBehaviour, current_behaviour)
        abci_app = current_behaviour.context.state.round_sequence.abci_app
        old_round = abci_app._current_round
        abci_app._last_round = old_round
        abci_app._current_round = abci_app.transition_function[
            current_behaviour.matching_round
        ][Event.DONE](abci_app.synchronized_data, abci_app.consensus_params)
        abci_app._previous_rounds.append(old_round)
        abci_app._current_round_height += 1
        self.hello_world_abci_behaviour._process_current_round()

    def _test_done_flag_set(self) -> None:
        """Test that, when round ends, the 'done' flag is set."""
        current_behaviour = cast(
            BaseBehaviour, self.hello_world_abci_behaviour.current_behaviour
        )
        assert not current_behaviour.is_done()
        with mock.patch.object(
            self.hello_world_abci_behaviour.context.state, "_round_sequence"
        ) as mock_round_sequence:
            mock_round_sequence.last_round_id = cast(
                AbstractRound, current_behaviour.matching_round
            ).round_id
            current_behaviour.act_wrapper()
            assert current_behaviour.is_done()

    @classmethod
    def teardown(cls) -> None:
        """Teardown the test class."""
        _MetaPayload.transaction_type_to_payload_cls = cls.old_tx_type_to_payload_cls  # type: ignore
        cls.benchmark_dir.cleanup()


class BaseSelectKeeperBehaviourTest(HelloWorldAbciFSMBehaviourBaseCase):
    """Test SelectKeeperBehaviour."""

    select_keeper_behaviour_class: Type[BaseBehaviour]
    next_behaviour_class: Type[BaseBehaviour]

    def test_select_keeper(
        self,
    ) -> None:
        """Test select keeper agent."""
        participants = frozenset({self.skill.skill_context.agent_address, "a_1", "a_2"})
        self.fast_forward_to_behaviour(
            behaviour=self.hello_world_abci_behaviour,
            behaviour_id=self.select_keeper_behaviour_class.behaviour_id,
            synchronized_data=SynchronizedData(
                AbciAppDB(
                    initial_data=dict(
                        participants=[participants],
                        most_voted_keeper_address=["most_voted_keeper_address"],
                    ),
                )
            ),
        )
        assert (
            cast(
                BaseBehaviour,
                cast(BaseBehaviour, self.hello_world_abci_behaviour.current_behaviour),
            ).behaviour_id
            == self.select_keeper_behaviour_class.behaviour_id
        )
        self.hello_world_abci_behaviour.act_wrapper()
        self.mock_a2a_transaction()
        self._test_done_flag_set()
        self.end_round()
        behaviour = cast(
            BaseBehaviour, self.hello_world_abci_behaviour.current_behaviour
        )
        assert behaviour.behaviour_id == self.next_behaviour_class.behaviour_id


class TestRegistrationBehaviour(HelloWorldAbciFSMBehaviourBaseCase):
    """Test case to test RegistrationBehaviour."""

    def test_registration(self) -> None:
        """Test registration."""
        self.fast_forward_to_behaviour(
            self.hello_world_abci_behaviour,
            RegistrationBehaviour.behaviour_id,
            self.synchronized_data,
        )
        assert (
            cast(
                BaseBehaviour,
                cast(BaseBehaviour, self.hello_world_abci_behaviour.current_behaviour),
            ).behaviour_id
            == RegistrationBehaviour.behaviour_id
        )
        self.hello_world_abci_behaviour.act_wrapper()
        self.mock_a2a_transaction()
        self._test_done_flag_set()

        self.end_round()
        behaviour = cast(
            BaseBehaviour, self.hello_world_abci_behaviour.current_behaviour
        )
        assert behaviour.behaviour_id == SelectKeeperBehaviour.behaviour_id


class TestSelectKeeperBehaviour(BaseSelectKeeperBehaviourTest):
    """Test SelectKeeperBehaviour."""

    select_keeper_behaviour_class = SelectKeeperBehaviour
    next_behaviour_class = PrintMessageBehaviour


class TestPrintMessageBehaviour(HelloWorldAbciFSMBehaviourBaseCase):
    """Test case to test PrintMessageBehaviour."""

    def test_print_message_non_keeper(self) -> None:
        """Test print_message."""
        self.fast_forward_to_behaviour(
            self.hello_world_abci_behaviour,
            PrintMessageBehaviour.behaviour_id,
            self.synchronized_data,
        )
        assert (
            cast(
                BaseBehaviour,
                cast(BaseBehaviour, self.hello_world_abci_behaviour.current_behaviour),
            ).behaviour_id
            == PrintMessageBehaviour.behaviour_id
        )
        self.hello_world_abci_behaviour.act_wrapper()
        self.mock_a2a_transaction()
        self._test_done_flag_set()

        self.end_round()
        behaviour = cast(
            BaseBehaviour, self.hello_world_abci_behaviour.current_behaviour
        )
        assert behaviour.behaviour_id == ResetAndPauseBehaviour.behaviour_id

    @mock.patch.object(SkillContext, "agent_address", new_callable=mock.PropertyMock)
    def test_print_message_keeper(
        self,
        agent_address_mock: mock.PropertyMock,
    ) -> None:
        """Test print_message."""
        agent_address_mock.return_value = "most_voted_keeper_address"
        self.fast_forward_to_behaviour(
            self.hello_world_abci_behaviour,
            PrintMessageBehaviour.behaviour_id,
            self.synchronized_data,
        )
        assert (
            cast(
                BaseBehaviour,
                cast(BaseBehaviour, self.hello_world_abci_behaviour.current_behaviour),
            ).behaviour_id
            == PrintMessageBehaviour.behaviour_id
        )
        self.hello_world_abci_behaviour.act_wrapper()
        self.mock_a2a_transaction()
        self._test_done_flag_set()

        self.end_round()
        behaviour = cast(
            BaseBehaviour, self.hello_world_abci_behaviour.current_behaviour
        )
        assert behaviour.behaviour_id == ResetAndPauseBehaviour.behaviour_id


class TestResetAndPauseBehaviour(HelloWorldAbciFSMBehaviourBaseCase):
    """Test ResetBehaviour."""

    behaviour_class = ResetAndPauseBehaviour
    next_behaviour_class = SelectKeeperBehaviour

    def test_pause_and_reset_behaviour(
        self,
    ) -> None:
        """Test pause and reset behaviour."""
        self.fast_forward_to_behaviour(
            behaviour=self.hello_world_abci_behaviour,
            behaviour_id=self.behaviour_class.behaviour_id,
            synchronized_data=self.synchronized_data,
        )
        assert (
            cast(
                BaseBehaviour,
                cast(BaseBehaviour, self.hello_world_abci_behaviour.current_behaviour),
            ).behaviour_id
            == self.behaviour_class.behaviour_id
        )
        self.hello_world_abci_behaviour.context.params.observation_interval = 0.1
        self.hello_world_abci_behaviour.act_wrapper()
        time.sleep(0.3)
        self.hello_world_abci_behaviour.act_wrapper()
        self.mock_a2a_transaction()
        self._test_done_flag_set()
        self.end_round()
        behaviour = cast(
            BaseBehaviour, self.hello_world_abci_behaviour.current_behaviour
        )
        assert behaviour.behaviour_id == self.next_behaviour_class.behaviour_id

    def test_reset_behaviour(
        self,
    ) -> None:
        """Test reset behaviour."""
        self.fast_forward_to_behaviour(
            behaviour=self.hello_world_abci_behaviour,
            behaviour_id=self.behaviour_class.behaviour_id,
            synchronized_data=self.synchronized_data,
        )
        self.hello_world_abci_behaviour.current_behaviour.pause = False  # type: ignore
        assert (
            cast(
                BaseBehaviour,
                cast(BaseBehaviour, self.hello_world_abci_behaviour.current_behaviour),
            ).behaviour_id
            == self.behaviour_class.behaviour_id
        )
        self.hello_world_abci_behaviour.act_wrapper()
        self.mock_a2a_transaction()
        self._test_done_flag_set()
        self.end_round()
        behaviour = cast(
            BaseBehaviour, self.hello_world_abci_behaviour.current_behaviour
        )
        assert behaviour.behaviour_id == self.next_behaviour_class.behaviour_id



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Script to build and run tendermint nodes from data dumps."""

import json
import os
import shutil
import signal
import subprocess  # nosec
import time
from pathlib import Path
from typing import Dict, List, Optional

from flask import Flask


class RanOutOfDumpsToReplay(Exception):
    """Error to raise when we run out of dumps to replay."""


class TendermintRunner:
    """Run tednermint using the dump."""

    node_id: int
    process: Optional[subprocess.Popen]  # nosec
    period: int = 0
    n_periods: int

    def __init__(self, node_id: int, dump_dir: Path, n_periods: int) -> None:
        """Initialize object."""

        self.period = 0
        self.process = None
        self.dump_dir = dump_dir
        self.node_id = node_id
        self.n_periods = n_periods

    def update_period(
        self,
    ) -> None:
        """Update period."""
        self.stop()
        self.period += 1
        if self.period >= self.n_periods:
            raise RanOutOfDumpsToReplay()
        self.start()

    def get_last_block_height(
        self,
    ) -> int:
        """Returns the last block height before dumping."""
        state_file = (
            self.dump_dir
            / f"period_{self.period}"
            / f"node{self.node_id}"
            / "data"
            / "priv_validator_state.json"
        )
        state_data = json.loads(state_file.read_text())
        return int(state_data.get("height", 0))

    def start(
        self,
    ) -> None:
        """Start tendermint process."""

        tendermint_bin = shutil.which("tendermint")
        if tendermint_bin is None:
            raise ValueError("Cannot find tendermint installation.")

        self.process = subprocess.Popen(  # nosec  # pylint: disable=subprocess-run-check,consider-using-with
            [
                str(tendermint_bin),
                "node",
                f"--p2p.laddr=tcp://127.0.0.1:2663{self.node_id}",
                f"--rpc.laddr=tcp://localhost:2664{self.node_id}",
                f"--proxy_app=tcp://localhost:2665{self.node_id}",
                "--consensus.create_empty_blocks=true",
                "--home",
                str(self.dump_dir / f"period_{self.period}" / f"node{self.node_id}"),
            ],
        )

    def stop(
        self,
    ) -> None:
        """Stop tendermint process."""
        if self.process is None:
            return

        self.process.poll()
        if self.process.returncode is None:  # stop only pending processes
            os.kill(self.process.pid, signal.SIGTERM)

        self.process.wait(timeout=5)
        self.process.terminate()
        self.process = None


class TendermintNetwork:
    """Tendermint network."""

    dump_dir: Path
    number_of_nodes: int
    nodes: List[TendermintRunner]
    reset_periods: int

    _recent_reset: bool

    def init(self, dump_dir: Path) -> None:
        """Initialize object."""

        self.dump_dir = dump_dir
        self.reset_periods = len(list(dump_dir.glob("period_*")))
        if self.reset_periods == 0:
            raise FileNotFoundError(f"Can't find period dumps in {dump_dir}")

        self.number_of_nodes = len(list((dump_dir / "period_0").iterdir()))
        if self.number_of_nodes == 0:
            raise FileNotFoundError("Can't find dumped nodes.")

        self.nodes = []
        self._recent_reset = True
        for node_id in range(self.number_of_nodes):
            self.nodes.append(
                TendermintRunner(node_id, self.dump_dir, self.reset_periods)
            )

    def update_period(self, node_id: int) -> None:
        """Update period for nth node."""
        self.nodes[node_id].update_period()
        self._recent_reset = True

    def get_last_block_height(self, node_id: int) -> int:
        """Returns last block height before dumping for `node_id`"""
        if self._recent_reset:
            self._recent_reset = False
            return 0

        return self.nodes[node_id].get_last_block_height() - 1

    def stop_node(self, node_id: int) -> None:
        """Stop a specific node."""
        self.nodes[node_id].stop()

    def start(
        self,
    ) -> None:
        """Start networks."""
        for node in self.nodes:
            node.start()

    def stop(
        self,
    ) -> None:
        """Stop network."""

        for node in self.nodes:
            node.stop()

    def run_until_interruption(
        self,
    ) -> None:
        """Run network until interruption."""

        try:
            self.start()
            while True:
                time.sleep(1)
        except KeyboardInterrupt:
            self.stop()


app = Flask(__name__)
tendermint_network = TendermintNetwork()


@app.route("/<int:node_id>/hard_reset")
def hard_reset(node_id: int) -> Dict:
    """Reset tendermint node."""
    global tendermint_network  # pylint: disable=global-variable-not-assigned

    try:
        tendermint_network.update_period(node_id)
        return {"message": "Reset successful.", "status": True, "is_replay": True}

    except RanOutOfDumpsToReplay:
        tendermint_network.stop_node(node_id)
        return {
            "message": "Ran out of dumps to replay, You can stop the agent replay now.",
            "status": False,
            "is_replay": True,
        }


@app.get("/<int:node_id>/status")
def status(node_id: int) -> Dict:
    """
    Status

    This endpoint will imitate the tendermint RPC server's /status so the ABCI
    app doesn't get blocked in replay mode.

    :param node_id: node id
    :return: response
    """

    global tendermint_network  # pylint: disable=global-variable-not-assigned

    return {
        "result": {
            "sync_info": {
                "latest_block_height": tendermint_network.get_last_block_height(node_id)
            }
        }
    }


@app.get("/<int:node_id>/broadcast_tx_sync")
def broadcast_tx_sync(node_id: int) -> Dict:  # pylint: disable=unused-argument
    """Similar as /status"""
    return {"result": {"hash": "", "code": 0}}


@app.get("/<int:node_id>/tx")
def tx(node_id: int) -> Dict:  # pylint: disable=unused-argument
    """Similar as /status"""
    return {"result": {"tx_result": {"code": 0}}}



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2021-2022 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Docker-compose Deployment Generator."""
import subprocess  # nosec
from pathlib import Path
from typing import Dict, IO, cast

from aea.configurations.constants import DEFAULT_PRIVATE_KEY_FILE

from autonomy.constants import (
    IMAGE_VERSION,
    OPEN_AEA_IMAGE_NAME,
    TENDERMINT_IMAGE_NAME,
    TENDERMINT_IMAGE_VERSION,
)
from autonomy.deploy.base import BaseDeploymentGenerator
from autonomy.deploy.constants import (
    DEFAULT_ENCODING,
    DEPLOYMENT_AGENT_KEY_DIRECTORY_SCHEMA,
    DEPLOYMENT_KEY_DIRECTORY,
)
from autonomy.deploy.generators.docker_compose.templates import (
    ABCI_NODE_TEMPLATE,
    DOCKER_COMPOSE_TEMPLATE,
    TENDERMINT_CONFIG_TEMPLATE,
    TENDERMINT_NODE_TEMPLATE,
)


def build_tendermint_node_config(
    node_id: int, dev_mode: bool = False, image_version: str = TENDERMINT_IMAGE_VERSION
) -> str:
    """Build tendermint node config for docker compose."""

    config = TENDERMINT_NODE_TEMPLATE.format(
        node_id=node_id,
        localnet_address_postfix=node_id + 3,
        localnet_port_range=node_id,
        tendermint_image_name=TENDERMINT_IMAGE_NAME,
        tendermint_image_version=image_version,
    )

    if dev_mode:
        config += "      - ./persistent_data/tm_state:/tm_state:Z"
        config = config.replace("DEV_MODE=0", "DEV_MODE=1")

    return config


def build_agent_config(  # pylint: disable=too-many-arguments
    valory_app: str,
    node_id: int,
    number_of_agents: int,
    agent_vars: Dict,
    dev_mode: bool = False,
    package_dir: Path = Path.cwd().absolute() / "packages",
    open_aea_dir: Path = Path.cwd().absolute().parent / "open-aea",
    open_aea_image_name: str = OPEN_AEA_IMAGE_NAME,
    open_aea_image_version: str = IMAGE_VERSION,
) -> str:
    """Build agent config."""

    config = ABCI_NODE_TEMPLATE.format(
        valory_app=valory_app,
        node_id=node_id,
        agent_vars="".join([f"      - {k}={v}\n" for k, v in agent_vars.items()]),
        localnet_address_postfix=node_id + number_of_agents + 3,
        open_aea_image_name=open_aea_image_name,
        open_aea_image_version=open_aea_image_version,
    )

    if dev_mode:
        config += "      - ./persistent_data/benchmarks:/benchmarks:Z\n"
        config += (
            "      - ./persistent_data/venvs:/home/ubuntu/.local/share/virtualenvs:Z\n"
        )
        config += f"      - {package_dir}:/home/ubuntu/packages:rw\n"
        config += f"      - {open_aea_dir}:/open-aea\n"

    return config


class DockerComposeGenerator(BaseDeploymentGenerator):
    """Class to automate the generation of Deployments."""

    output_name: str = "docker-compose.yaml"
    deployment_type: str = "docker-compose"

    def generate_config_tendermint(
        self, image_version: str = TENDERMINT_IMAGE_NAME
    ) -> "DockerComposeGenerator":
        """Generate the command to configure tendermint testnet."""

        if self.tendermint_job_config is not None:
            return self

        run_cmd = TENDERMINT_CONFIG_TEMPLATE.format(
            hosts=" \\\n".join(
                [
                    f"--hostname=node{k}"
                    for k in range(self.service_spec.service.number_of_agents)
                ]
            ),
            validators=self.service_spec.service.number_of_agents,
            build_dir=self.build_dir,
            tendermint_image_name=TENDERMINT_IMAGE_NAME,
            tendermint_image_version=image_version,
        )
        self.tendermint_job_config = " ".join(
            [
                f
                for f in run_cmd.replace("\n", "").replace("\\", "").split(" ")
                if f != ""
            ]
        )

        process = subprocess.Popen(  # pylint: disable=consider-using-with  # nosec
            self.tendermint_job_config.split(),
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            universal_newlines=True,
        )

        for line in iter(cast(IO[str], process.stdout).readline, ""):
            if line == "":
                break
            print(f"[Tendermint] {line.strip()}")

        if "Unable to find image" in cast(IO[str], process.stderr).read():
            raise RuntimeError(
                f"Cannot find {TENDERMINT_IMAGE_NAME}:{image_version}, Please build images first."
            )
        return self

    def generate(
        self,
        image_versions: Dict[str, str],
        dev_mode: bool = False,
    ) -> "DockerComposeGenerator":
        """Generate the new configuration."""

        agent_vars = self.service_spec.generate_agents()
        agent_vars = self.get_deployment_network_configuration(agent_vars)
        image_name = self.service_spec.service.agent.name

        if dev_mode:
            image_versions["agent"] = "dev"

        agents = "".join(
            [
                build_agent_config(
                    image_name,
                    i,
                    self.service_spec.service.number_of_agents,
                    agent_vars[i],
                    dev_mode,
                    open_aea_image_version=image_versions["agent"],
                )
                for i in range(self.service_spec.service.number_of_agents)
            ]
        )
        tendermint_nodes = "".join(
            [
                build_tendermint_node_config(i, dev_mode, image_versions["tendermint"])
                for i in range(self.service_spec.service.number_of_agents)
            ]
        )
        self.output = DOCKER_COMPOSE_TEMPLATE.format(
            abci_nodes=agents,
            tendermint_nodes=tendermint_nodes,
        )

        return self

    def populate_private_keys(
        self,
    ) -> "DockerComposeGenerator":
        """Populate the private keys to the build directory for docker-compose mapping."""
        for x in range(self.service_spec.service.number_of_agents):
            path = (
                self.build_dir
                / DEPLOYMENT_KEY_DIRECTORY
                / DEPLOYMENT_AGENT_KEY_DIRECTORY_SCHEMA.format(agent_n=x)
            )
            path.mkdir()
            with open(
                path / DEFAULT_PRIVATE_KEY_FILE, "w", encoding=DEFAULT_ENCODING
            ) as f:
                f.write(str(self.service_spec.private_keys[x]))
        return self



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2021-2022 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Test the rounds of the skill."""
import logging  # noqa: F401
import sys
from types import MappingProxyType
from typing import Dict, FrozenSet, cast
from unittest import mock

import atheris  # type: ignore
import pytest

from packages.valory.skills.abstract_round_abci.base import (
    AbciAppDB,
    AbstractRound,
    ConsensusParams,
    MAX_INT_256,
)
from packages.valory.skills.simple_abci.payloads import (
    RandomnessPayload,
    RegistrationPayload,
    ResetPayload,
    SelectKeeperPayload,
)
from packages.valory.skills.simple_abci.rounds import (
    Event,
    RandomnessStartupRound,
    RegistrationRound,
    ResetAndPauseRound,
    SelectKeeperAtStartupRound,
    SynchronizedData,
    rotate_list,
)


MAX_PARTICIPANTS: int = 4
RANDOMNESS: str = "d1c29dce46f979f9748210d24bce4eae8be91272f5ca1a6aea2832d3dd676f51"


def get_participants() -> FrozenSet[str]:
    """Participants"""
    return frozenset([f"agent_{i}" for i in range(MAX_PARTICIPANTS)])


def get_participant_to_randomness(
    participants: FrozenSet[str], round_id: int
) -> Dict[str, RandomnessPayload]:
    """participant_to_randomness"""
    return {
        participant: RandomnessPayload(
            sender=participant,
            round_id=round_id,
            randomness=RANDOMNESS,
        )
        for participant in participants
    }


def get_participant_to_selection(
    participants: FrozenSet[str],
) -> Dict[str, SelectKeeperPayload]:
    """participant_to_selection"""
    return {
        participant: SelectKeeperPayload(sender=participant, keeper="keeper")
        for participant in participants
    }


def get_participant_to_period_count(
    participants: FrozenSet[str], period_count: int
) -> Dict[str, ResetPayload]:
    """participant_to_selection"""
    return {
        participant: ResetPayload(sender=participant, period_count=period_count)
        for participant in participants
    }


class BaseRoundTestClass:
    """Base test class for Rounds."""

    synchronized_data: SynchronizedData
    consensus_params: ConsensusParams
    participants: FrozenSet[str]

    @classmethod
    def setup(
        cls,
    ) -> None:
        """Setup the test class."""

        cls.participants = get_participants()
        cls.synchronized_data = SynchronizedData(
            AbciAppDB(
                initial_data=dict(
                    participants=[cls.participants], all_participants=[cls.participants]
                ),
            )
        )
        cls.consensus_params = ConsensusParams(max_participants=MAX_PARTICIPANTS)

    def _test_no_majority_event(self, round_obj: AbstractRound) -> None:
        """Test the NO_MAJORITY event."""
        with mock.patch.object(round_obj, "is_majority_possible", return_value=False):
            result = round_obj.end_block()
            assert result is not None
            synchronized_data, event = result
            assert event == Event.NO_MAJORITY


class TestRegistrationRound(BaseRoundTestClass):
    """Tests for RegistrationRound."""

    def test_run(
        self,
    ) -> None:
        """Run tests."""

        test_round = RegistrationRound(
            synchronized_data=self.synchronized_data,
            consensus_params=self.consensus_params,
        )

        first_payload, *payloads = [
            RegistrationPayload(sender=participant) for participant in self.participants
        ]

        test_round.process_payload(first_payload)
        assert test_round.collection == {first_payload.sender: first_payload}
        assert test_round.end_block() is None

        for payload in payloads:
            test_round.process_payload(payload)

        actual_next_behaviour = SynchronizedData(
            AbciAppDB(initial_data=dict(participants=[test_round.collection]))
        )

        res = test_round.end_block()
        assert res is not None
        synchronized_data, event = res
        assert (
            cast(SynchronizedData, synchronized_data).participants
            == cast(SynchronizedData, actual_next_behaviour).participants
        )
        assert event == Event.DONE


class TestRandomnessStartupRound(BaseRoundTestClass):
    """Tests for RandomnessStartupRound."""

    def test_run(
        self,
    ) -> None:
        """Run tests."""

        test_round = RandomnessStartupRound(
            synchronized_data=self.synchronized_data,
            consensus_params=self.consensus_params,
        )
        first_payload, *payloads = [
            RandomnessPayload(sender=participant, randomness=RANDOMNESS, round_id=0)
            for participant in self.participants
        ]

        test_round.process_payload(first_payload)
        assert test_round.collection[first_payload.sender] == first_payload
        assert test_round.end_block() is None

        self._test_no_majority_event(test_round)

        for payload in payloads:
            test_round.process_payload(payload)

        actual_next_behaviour = self.synchronized_data.update(
            participant_to_randomness=MappingProxyType(test_round.collection),
            most_voted_randomness=test_round.most_voted_payload,
        )

        res = test_round.end_block()
        assert res is not None
        synchronized_data, event = res
        assert all(
            [
                key
                in cast(SynchronizedData, synchronized_data).participant_to_randomness
                for key in cast(
                    SynchronizedData, actual_next_behaviour
                ).participant_to_randomness
            ]
        )
        assert event == Event.DONE


class TestSelectKeeperAtStartupRound(BaseRoundTestClass):
    """Tests for SelectKeeperAtStartupRound."""

    def test_run(
        self,
    ) -> None:
        """Run tests."""

        test_round = SelectKeeperAtStartupRound(
            synchronized_data=self.synchronized_data,
            consensus_params=self.consensus_params,
        )

        first_payload, *payloads = [
            SelectKeeperPayload(sender=participant, keeper="keeper")
            for participant in self.participants
        ]

        test_round.process_payload(first_payload)
        assert test_round.collection[first_payload.sender] == first_payload
        assert test_round.end_block() is None

        self._test_no_majority_event(test_round)

        for payload in payloads:
            test_round.process_payload(payload)

        actual_next_behaviour = self.synchronized_data.update(
            participant_to_selection=MappingProxyType(test_round.collection),
            most_voted_keeper_address=test_round.most_voted_payload,
        )

        res = test_round.end_block()
        assert res is not None
        synchronized_data, event = res
        assert all(
            [
                key
                in cast(SynchronizedData, synchronized_data).participant_to_selection
                for key in cast(
                    SynchronizedData, actual_next_behaviour
                ).participant_to_selection
            ]
        )
        assert event == Event.DONE


class TestResetAndPauseRound(BaseRoundTestClass):
    """Tests for ResetAndPauseRound."""

    def test_run(
        self,
    ) -> None:
        """Run tests."""
        test_round = ResetAndPauseRound(
            synchronized_data=self.synchronized_data,
            consensus_params=self.consensus_params,
        )

        first_payload, *payloads = [
            ResetPayload(sender=participant, period_count=1)
            for participant in self.participants
        ]

        test_round.process_payload(first_payload)
        assert test_round.collection[first_payload.sender] == first_payload
        assert test_round.end_block() is None

        self._test_no_majority_event(test_round)

        for payload in payloads:
            test_round.process_payload(payload)

        actual_next_behaviour = self.synchronized_data.create(
            participants=[self.synchronized_data.participants],
            all_participants=[self.synchronized_data.all_participants],
        )

        res = test_round.end_block()
        assert res is not None
        synchronized_data, event = res

        assert (
            cast(SynchronizedData, synchronized_data).period_count
            == cast(SynchronizedData, actual_next_behaviour).period_count
        )

        assert event == Event.DONE


def test_rotate_list_method() -> None:
    """Test `rotate_list` method."""

    ex_list = [1, 2, 3, 4, 5]
    assert rotate_list(ex_list, 2) == [3, 4, 5, 1, 2]


@pytest.mark.skip
def test_fuzz_rotate_list() -> None:
    """Test fuzz rotate_list."""

    @atheris.instrument_func
    def fuzz_rotate_list(input_bytes: bytes) -> None:
        """Fuzz rotate_list."""
        fdp = atheris.FuzzedDataProvider(input_bytes)
        elements = [fdp.ConsumeString(4) for _ in range(10)]
        positions = fdp.ConsumeInt(4)
        rotate_list(elements, positions)

    atheris.instrument_all()
    atheris.Setup(sys.argv, fuzz_rotate_list)
    atheris.Fuzz()


def test_synchronized_data() -> None:  # pylint:too-many-locals
    """Test SynchronizedData."""

    participants = get_participants()
    setup_params = {}  # type: ignore
    participant_to_randomness = {
        participant: RandomnessPayload(
            sender=participant, randomness=RANDOMNESS, round_id=0
        )
        for participant in participants
    }
    most_voted_randomness = "0xabcd"
    participant_to_selection = {
        participant: SelectKeeperPayload(sender=participant, keeper="keeper")
        for participant in participants
    }
    most_voted_keeper_address = "keeper"

    synchronized_data = SynchronizedData(
        AbciAppDB(
            initial_data=AbciAppDB.data_to_lists(
                dict(
                    participants=participants,
                    setup_params=setup_params,
                    participant_to_randomness=participant_to_randomness,
                    most_voted_randomness=most_voted_randomness,
                    participant_to_selection=participant_to_selection,
                    most_voted_keeper_address=most_voted_keeper_address,
                )
            ),
        )
    )

    assert synchronized_data.participants == participants
    assert synchronized_data.period_count == 0
    assert synchronized_data.participant_to_randomness == participant_to_randomness
    assert synchronized_data.most_voted_randomness == most_voted_randomness
    assert synchronized_data.participant_to_selection == participant_to_selection
    assert synchronized_data.most_voted_keeper_address == most_voted_keeper_address
    assert synchronized_data.sorted_participants == sorted(participants)
    actual_keeper_randomness = int(most_voted_randomness, base=16) / MAX_INT_256
    assert (
        abs(synchronized_data.keeper_randomness - actual_keeper_randomness) < 1e-10
    )  # avoid equality comparisons between floats



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2021-2022 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""This module contains the behaviours for the 'hello_world' skill."""

from abc import ABC
from typing import Generator, Set, Type, cast

from packages.valory.skills.abstract_round_abci.behaviours import (
    AbstractRoundBehaviour,
    BaseBehaviour,
)
from packages.valory.skills.hello_world_abci.models import Params, SharedState
from packages.valory.skills.hello_world_abci.payloads import (
    PrintMessagePayload,
    RegistrationPayload,
    ResetPayload,
    SelectKeeperPayload,
)
from packages.valory.skills.hello_world_abci.rounds import (
    HelloWorldAbciApp,
    PrintMessageRound,
    RegistrationRound,
    ResetAndPauseRound,
    SelectKeeperRound,
    SynchronizedData,
)


class HelloWorldABCIBaseBehaviour(BaseBehaviour, ABC):
    """Base behaviour behaviour for the Hello World abci skill."""

    @property
    def synchronized_data(self) -> SynchronizedData:
        """Return the synchronized data."""
        return cast(
            SynchronizedData, cast(SharedState, self.context.state).synchronized_data
        )

    @property
    def params(self) -> Params:
        """Return the params."""
        return cast(Params, self.context.params)


class RegistrationBehaviour(HelloWorldABCIBaseBehaviour):
    """Register to the next round."""

    behaviour_id = "register"
    matching_round = RegistrationRound

    def async_act(self) -> Generator:
        """
        Do the action.

        Steps:
        - Build a registration transaction.
        - Send the transaction and wait for it to be mined.
        - Wait until ABCI application transitions to the next round.
        - Go to the next behaviour (set done event).
        """

        with self.context.benchmark_tool.measure(self.behaviour_id).local():
            payload = RegistrationPayload(self.context.agent_address)

        with self.context.benchmark_tool.measure(self.behaviour_id).consensus():
            yield from self.send_a2a_transaction(payload)
            yield from self.wait_until_round_end()

        self.set_done()


class SelectKeeperBehaviour(HelloWorldABCIBaseBehaviour, ABC):
    """Select the keeper agent."""

    behaviour_id = "select_keeper"
    matching_round = SelectKeeperRound

    def async_act(self) -> Generator:
        """
        Do the action.

        Steps:
        - Select a keeper randomly.
        - Send the transaction with the keeper and wait for it to be mined.
        - Wait until ABCI application transitions to the next round.
        - Go to the next behaviour (set done event).
        """

        with self.context.benchmark_tool.measure(self.behaviour_id).local():
            keeper_address = sorted(self.synchronized_data.participants)[
                self.synchronized_data.period_count
                % self.synchronized_data.nb_participants
            ]

            self.context.logger.info(f"Selected a new keeper: {keeper_address}.")
            payload = SelectKeeperPayload(self.context.agent_address, keeper_address)

        with self.context.benchmark_tool.measure(self.behaviour_id).consensus():
            yield from self.send_a2a_transaction(payload)
            yield from self.wait_until_round_end()

        self.set_done()


class PrintMessageBehaviour(HelloWorldABCIBaseBehaviour, ABC):
    """Prints the celebrated 'HELLO WORLD!' message."""

    behaviour_id = "print_message"
    matching_round = PrintMessageRound

    def async_act(self) -> Generator:
        """
        Do the action.

        Steps:
        - Determine if this agent is the current keeper agent.
        - Print the appropriate to the local console.
        - Send the transaction with the printed message and wait for it to be mined.
        - Wait until ABCI application transitions to the next round.
        - Go to the next behaviour (set done event).
        """

        printed_message = f"Agent {self.context.agent_name} (address {self.context.agent_address}) in period {self.synchronized_data.period_count} says: "
        if (
            self.context.agent_address
            == self.synchronized_data.most_voted_keeper_address
        ):
            printed_message += "HELLO WORLD!"
        else:
            printed_message += ":|"

        print(printed_message)
        self.context.logger.info(f"printed_message={printed_message}")

        payload = PrintMessagePayload(self.context.agent_address, printed_message)

        with self.context.benchmark_tool.measure(self.behaviour_id).consensus():
            yield from self.send_a2a_transaction(payload)
            yield from self.wait_until_round_end()

        self.set_done()


class ResetAndPauseBehaviour(HelloWorldABCIBaseBehaviour):
    """Reset behaviour."""

    matching_round = ResetAndPauseRound
    behaviour_id = "reset_and_pause"
    pause = True

    pause = True

    def async_act(self) -> Generator:
        """
        Do the action.

        Steps:
        - Trivially log the behaviour.
        - Sleep for configured interval.
        - Build a registration transaction.
        - Send the transaction and wait for it to be mined.
        - Wait until ABCI application transitions to the next round.
        - Go to the next behaviour (set done event).
        """
        if self.pause:
            self.context.logger.info("Period end.")
            self.context.benchmark_tool.save()
            yield from self.sleep(self.params.observation_interval)
        else:
            self.context.logger.info(
                f"Period {self.synchronized_data.period_count} was not finished. Resetting!"
            )

        payload = ResetPayload(
            self.context.agent_address, self.synchronized_data.period_count
        )

        yield from self.send_a2a_transaction(payload)
        yield from self.wait_until_round_end()
        self.set_done()


class HelloWorldRoundBehaviour(AbstractRoundBehaviour):
    """This behaviour manages the consensus stages for the Hello World abci app."""

    initial_behaviour_cls = RegistrationBehaviour
    abci_app_cls = HelloWorldAbciApp  # type: ignore
    behaviours: Set[Type[HelloWorldABCIBaseBehaviour]] = {  # type: ignore
        RegistrationBehaviour,  # type: ignore
        SelectKeeperBehaviour,  # type: ignore
        PrintMessageBehaviour,  # type: ignore
        ResetAndPauseBehaviour,  # type: ignore
    }



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Tools to build and run agents from existing deployments."""

import os
import signal
import subprocess  # nosec
import sys
from pathlib import Path
from tempfile import TemporaryDirectory
from typing import Dict, List, Optional


class AgentRunner:
    """Agent runner."""

    agent_id: int
    agent_data: Dict[str, str]
    registry_path: Path
    process: Optional[subprocess.Popen]  # nosec
    aea_cli: List[str] = [sys.executable, "-m", "aea.cli"]

    def __init__(self, agent_id: int, agent_data: Dict, registry_path: Path) -> None:
        """Initialize object."""

        self.agent_id = agent_id
        self.agent_data = agent_data
        self.registry_path = registry_path
        self.agent_env = os.environ.copy()
        self.agent_dir = TemporaryDirectory()  # pylint: disable=consider-using-with
        self.cwd = Path(".").resolve().absolute()

        agent_env_data = self.agent_data["environment"]
        for env_var in agent_env_data:
            key, value = env_var.split("=")
            self.agent_env[key] = value

        self.agent_env["ABCI_HOST"] = "localhost"
        self.agent_env["ABCI_PORT"] = f"2665{self.agent_id}"
        self.agent_env["TENDERMINT_URL"] = f"http://localhost:8080/{self.agent_id}"
        self.agent_env["TENDERMINT_COM_URL"] = f"http://localhost:8080/{self.agent_id}"

    def start(
        self,
    ) -> None:
        """Start process."""

        agent_dir = Path(self.agent_dir.name)
        os.chdir(agent_dir)

        print(f"Loading {self.agent_env['VALORY_APPLICATION']}")
        subprocess.run(  # nosec  # pylint: disable=subprocess-run-check
            [
                *self.aea_cli,
                "--registry-path",
                str(self.registry_path),
                "fetch",
                self.agent_env["VALORY_APPLICATION"],
                "--local",
                "--alias",
                "agent",
            ],
            env=self.agent_env,
        )
        os.chdir(agent_dir / "agent")
        Path(agent_dir, "agent", "ethereum_private_key.txt").write_text(
            self.agent_env["AEA_KEY"], encoding="utf-8"
        )

        subprocess.run(  # nosec # pylint: disable=subprocess-run-check
            [*self.aea_cli, "add-key", "ethereum"], env=self.agent_env
        )
        subprocess.run(  # nosec # pylint: disable=subprocess-run-check
            [*self.aea_cli, "install"], env=self.agent_env
        )
        self.process = subprocess.Popen(  # nosec # pylint: disable=subprocess-run-check,consider-using-with
            [*self.aea_cli, "run", "--aev"], env=self.agent_env
        )

    def stop(
        self,
    ) -> None:
        """Stop the process."""
        self.agent_dir.cleanup()
        os.chdir(str(self.cwd))
        if self.process is None:
            return

        os.kill(os.getpgid(self.process.pid), signal.SIGTERM)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2021-2022 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""This module contains the data classes for the simple ABCI application."""
from abc import ABC
from enum import Enum
from typing import Dict, Optional, Tuple, Type

from packages.valory.skills.abstract_round_abci.base import (
    AbciApp,
    AbciAppTransitionFunction,
    AbstractRound,
    BaseSynchronizedData,
    CollectDifferentUntilAllRound,
)
from packages.valory.skills.test_abci.payloads import DummyPayload, TransactionType


class Event(Enum):
    """Event enumeration for the simple abci demo."""

    DONE = "done"
    ROUND_TIMEOUT = "round_timeout"
    RESET_TIMEOUT = "reset_timeout"


class DummyRound(
    CollectDifferentUntilAllRound, AbstractRound[Event, TransactionType], ABC
):
    """
    This class represents the registration round.

    Input: None
    Output: a synchronized data with the set of participants.

    It schedules the SelectKeeperARound.
    """

    round_id = "dummy"
    allowed_tx_type = DummyPayload.transaction_type
    payload_attribute = "sender"

    def end_block(self) -> Optional[Tuple[BaseSynchronizedData, Event]]:
        """Process the end of the block."""
        return self.synchronized_data, Event.DONE


class TestAbciApp(AbciApp[Event]):
    """TestAbciApp

    Initial round: DummyRound

    Initial states: {DummyRound}

    Transition states:
        0. DummyRound
            - done: 0.

    Final states: {}

    Timeouts:
        round timeout: 30.0
        reset timeout: 30.0
    """

    initial_round_cls: Type[AbstractRound] = DummyRound
    transition_function: AbciAppTransitionFunction = {
        DummyRound: {Event.DONE: DummyRound}
    }
    event_to_timeout: Dict[Event, float] = {
        Event.ROUND_TIMEOUT: 30.0,
        Event.RESET_TIMEOUT: 30.0,
    }



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: abci.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database


# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\nabci.proto\x12\x16\x61\x65\x61.valory.abci.v0_1_0"\x86P\n\x0b\x41\x62\x63iMessage\x12G\n\x05\x64ummy\x18\x05 \x01(\x0b\x32\x36.aea.valory.abci.v0_1_0.AbciMessage.Dummy_PerformativeH\x00\x12u\n\x1crequest_apply_snapshot_chunk\x18\x06 \x01(\x0b\x32M.aea.valory.abci.v0_1_0.AbciMessage.Request_Apply_Snapshot_Chunk_PerformativeH\x00\x12\x63\n\x13request_begin_block\x18\x07 \x01(\x0b\x32\x44.aea.valory.abci.v0_1_0.AbciMessage.Request_Begin_Block_PerformativeH\x00\x12]\n\x10request_check_tx\x18\x08 \x01(\x0b\x32\x41.aea.valory.abci.v0_1_0.AbciMessage.Request_Check_Tx_PerformativeH\x00\x12Y\n\x0erequest_commit\x18\t \x01(\x0b\x32?.aea.valory.abci.v0_1_0.AbciMessage.Request_Commit_PerformativeH\x00\x12\x61\n\x12request_deliver_tx\x18\n \x01(\x0b\x32\x43.aea.valory.abci.v0_1_0.AbciMessage.Request_Deliver_Tx_PerformativeH\x00\x12U\n\x0crequest_echo\x18\x0b \x01(\x0b\x32=.aea.valory.abci.v0_1_0.AbciMessage.Request_Echo_PerformativeH\x00\x12_\n\x11request_end_block\x18\x0c \x01(\x0b\x32\x42.aea.valory.abci.v0_1_0.AbciMessage.Request_End_Block_PerformativeH\x00\x12W\n\rrequest_flush\x18\r \x01(\x0b\x32>.aea.valory.abci.v0_1_0.AbciMessage.Request_Flush_PerformativeH\x00\x12U\n\x0crequest_info\x18\x0e \x01(\x0b\x32=.aea.valory.abci.v0_1_0.AbciMessage.Request_Info_PerformativeH\x00\x12\x61\n\x12request_init_chain\x18\x0f \x01(\x0b\x32\x43.aea.valory.abci.v0_1_0.AbciMessage.Request_Init_Chain_PerformativeH\x00\x12i\n\x16request_list_snapshots\x18\x10 \x01(\x0b\x32G.aea.valory.abci.v0_1_0.AbciMessage.Request_List_Snapshots_PerformativeH\x00\x12s\n\x1brequest_load_snapshot_chunk\x18\x11 \x01(\x0b\x32L.aea.valory.abci.v0_1_0.AbciMessage.Request_Load_Snapshot_Chunk_PerformativeH\x00\x12i\n\x16request_offer_snapshot\x18\x12 \x01(\x0b\x32G.aea.valory.abci.v0_1_0.AbciMessage.Request_Offer_Snapshot_PerformativeH\x00\x12W\n\rrequest_query\x18\x13 \x01(\x0b\x32>.aea.valory.abci.v0_1_0.AbciMessage.Request_Query_PerformativeH\x00\x12\x61\n\x12request_set_option\x18\x14 \x01(\x0b\x32\x43.aea.valory.abci.v0_1_0.AbciMessage.Request_Set_Option_PerformativeH\x00\x12w\n\x1dresponse_apply_snapshot_chunk\x18\x15 \x01(\x0b\x32N.aea.valory.abci.v0_1_0.AbciMessage.Response_Apply_Snapshot_Chunk_PerformativeH\x00\x12\x65\n\x14response_begin_block\x18\x16 \x01(\x0b\x32\x45.aea.valory.abci.v0_1_0.AbciMessage.Response_Begin_Block_PerformativeH\x00\x12_\n\x11response_check_tx\x18\x17 \x01(\x0b\x32\x42.aea.valory.abci.v0_1_0.AbciMessage.Response_Check_Tx_PerformativeH\x00\x12[\n\x0fresponse_commit\x18\x18 \x01(\x0b\x32@.aea.valory.abci.v0_1_0.AbciMessage.Response_Commit_PerformativeH\x00\x12\x63\n\x13response_deliver_tx\x18\x19 \x01(\x0b\x32\x44.aea.valory.abci.v0_1_0.AbciMessage.Response_Deliver_Tx_PerformativeH\x00\x12W\n\rresponse_echo\x18\x1a \x01(\x0b\x32>.aea.valory.abci.v0_1_0.AbciMessage.Response_Echo_PerformativeH\x00\x12\x61\n\x12response_end_block\x18\x1b \x01(\x0b\x32\x43.aea.valory.abci.v0_1_0.AbciMessage.Response_End_Block_PerformativeH\x00\x12\x61\n\x12response_exception\x18\x1c \x01(\x0b\x32\x43.aea.valory.abci.v0_1_0.AbciMessage.Response_Exception_PerformativeH\x00\x12Y\n\x0eresponse_flush\x18\x1d \x01(\x0b\x32?.aea.valory.abci.v0_1_0.AbciMessage.Response_Flush_PerformativeH\x00\x12W\n\rresponse_info\x18\x1e \x01(\x0b\x32>.aea.valory.abci.v0_1_0.AbciMessage.Response_Info_PerformativeH\x00\x12\x63\n\x13response_init_chain\x18\x1f \x01(\x0b\x32\x44.aea.valory.abci.v0_1_0.AbciMessage.Response_Init_Chain_PerformativeH\x00\x12k\n\x17response_list_snapshots\x18  \x01(\x0b\x32H.aea.valory.abci.v0_1_0.AbciMessage.Response_List_Snapshots_PerformativeH\x00\x12u\n\x1cresponse_load_snapshot_chunk\x18! \x01(\x0b\x32M.aea.valory.abci.v0_1_0.AbciMessage.Response_Load_Snapshot_Chunk_PerformativeH\x00\x12k\n\x17response_offer_snapshot\x18" \x01(\x0b\x32H.aea.valory.abci.v0_1_0.AbciMessage.Response_Offer_Snapshot_PerformativeH\x00\x12Y\n\x0eresponse_query\x18# \x01(\x0b\x32?.aea.valory.abci.v0_1_0.AbciMessage.Response_Query_PerformativeH\x00\x12\x63\n\x13response_set_option\x18$ \x01(\x0b\x32\x44.aea.valory.abci.v0_1_0.AbciMessage.Response_Set_Option_PerformativeH\x00\x1a\x7f\n\x0b\x43heckTxType\x12J\n\x04type\x18\x01 \x01(\x0e\x32<.aea.valory.abci.v0_1_0.AbciMessage.CheckTxType._CheckTxType"$\n\x0c_CheckTxType\x12\x07\n\x03NEW\x10\x00\x12\x0b\n\x07RECHECK\x10\x01\x1a\xac\x05\n\x0f\x43onsensusParams\x12N\n\x05\x62lock\x18\x01 \x01(\x0b\x32?.aea.valory.abci.v0_1_0.AbciMessage.ConsensusParams.BlockParams\x12T\n\x08\x65vidence\x18\x02 \x01(\x0b\x32\x42.aea.valory.abci.v0_1_0.AbciMessage.ConsensusParams.EvidenceParams\x12V\n\tvalidator\x18\x03 \x01(\x0b\x32\x43.aea.valory.abci.v0_1_0.AbciMessage.ConsensusParams.ValidatorParams\x12R\n\x07version\x18\x04 \x01(\x0b\x32\x41.aea.valory.abci.v0_1_0.AbciMessage.ConsensusParams.VersionParams\x1a*\n\x08\x44uration\x12\x0f\n\x07seconds\x18\x01 \x01(\x03\x12\r\n\x05nanos\x18\x02 \x01(\x05\x1a\x31\n\x0b\x42lockParams\x12\x11\n\tmax_bytes\x18\x01 \x01(\x03\x12\x0f\n\x07max_gas\x18\x02 \x01(\x03\x1a\x97\x01\n\x0e\x45videnceParams\x12\x1a\n\x12max_age_num_blocks\x18\x01 \x01(\x03\x12V\n\x10max_age_duration\x18\x02 \x01(\x0b\x32<.aea.valory.abci.v0_1_0.AbciMessage.ConsensusParams.Duration\x12\x11\n\tmax_bytes\x18\x03 \x01(\x03\x1a(\n\x0fValidatorParams\x12\x15\n\rpub_key_types\x18\x01 \x03(\t\x1a$\n\rVersionParams\x12\x13\n\x0b\x61pp_version\x18\x01 \x01(\x04\x1a\xed\x01\n\x06\x45vents\x12@\n\x06\x65vents\x18\x01 \x03(\x0b\x32\x30.aea.valory.abci.v0_1_0.AbciMessage.Events.Event\x1a;\n\x0e\x45ventAttribute\x12\x0b\n\x03key\x18\x01 \x01(\x0c\x12\r\n\x05value\x18\x02 \x01(\x0c\x12\r\n\x05index\x18\x03 \x01(\x08\x1a\x64\n\x05\x45vent\x12\x0c\n\x04type\x18\x01 \x01(\t\x12M\n\nattributes\x18\x02 \x03(\x0b\x32\x39.aea.valory.abci.v0_1_0.AbciMessage.Events.EventAttribute\x1a\xc5\x03\n\tEvidences\x12T\n\x14\x62yzantine_validators\x18\x01 \x03(\x0b\x32\x36.aea.valory.abci.v0_1_0.AbciMessage.Evidences.Evidence\x1a\xe1\x02\n\x08\x45vidence\x12Q\n\x04type\x18\x01 \x01(\x0e\x32\x43.aea.valory.abci.v0_1_0.AbciMessage.Evidences.Evidence.EvidenceType\x12O\n\tvalidator\x18\x02 \x01(\x0b\x32<.aea.valory.abci.v0_1_0.AbciMessage.LastCommitInfo.Validator\x12\x0e\n\x06height\x18\x03 \x01(\x03\x12;\n\x04time\x18\x04 \x01(\x0b\x32-.aea.valory.abci.v0_1_0.AbciMessage.Timestamp\x12\x1a\n\x12total_voting_power\x18\x05 \x01(\x03"H\n\x0c\x45videnceType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x12\n\x0e\x44UPLICATE_VOTE\x10\x01\x12\x17\n\x13LIGHT_CLIENT_ATTACK\x10\x02\x1a\xa4\x05\n\x06Header\x12L\n\x07version\x18\x01 \x01(\x0b\x32;.aea.valory.abci.v0_1_0.AbciMessage.Header.ConsensusVersion\x12\x10\n\x08\x63hain_id\x18\x02 \x01(\t\x12\x0e\n\x06height\x18\x03 \x01(\x03\x12;\n\x04time\x18\x04 \x01(\x0b\x32-.aea.valory.abci.v0_1_0.AbciMessage.Timestamp\x12I\n\rlast_block_id\x18\x05 \x01(\x0b\x32\x32.aea.valory.abci.v0_1_0.AbciMessage.Header.BlockID\x12\x18\n\x10last_commit_hash\x18\x06 \x01(\x0c\x12\x11\n\tdata_hash\x18\x07 \x01(\x0c\x12\x17\n\x0fvalidators_hash\x18\x08 \x01(\x0c\x12\x1c\n\x14next_validators_hash\x18\t \x01(\x0c\x12\x16\n\x0e\x63onsensus_hash\x18\n \x01(\x0c\x12\x10\n\x08\x61pp_hash\x18\x0b \x01(\x0c\x12\x19\n\x11last_results_hash\x18\x0c \x01(\x0c\x12\x15\n\revidence_hash\x18\r \x01(\x0c\x12\x18\n\x10proposer_address\x18\x0e \x01(\x0c\x1a.\n\x10\x43onsensusVersion\x12\r\n\x05\x62lock\x18\x01 \x01(\x04\x12\x0b\n\x03\x61pp\x18\x02 \x01(\x04\x1aj\n\x07\x42lockID\x12\x0c\n\x04hash\x18\x01 \x01(\x0c\x12Q\n\x0fpart_set_header\x18\x02 \x01(\x0b\x32\x38.aea.valory.abci.v0_1_0.AbciMessage.Header.PartSetHeader\x1a,\n\rPartSetHeader\x12\r\n\x05total\x18\x01 \x01(\r\x12\x0c\n\x04hash\x18\x02 \x01(\x0c\x1a\x90\x02\n\x0eLastCommitInfo\x12\r\n\x05round\x18\x01 \x01(\x05\x12J\n\x05votes\x18\x02 \x03(\x0b\x32;.aea.valory.abci.v0_1_0.AbciMessage.LastCommitInfo.VoteInfo\x1a+\n\tValidator\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\x0c\x12\r\n\x05power\x18\x03 \x01(\x03\x1av\n\x08VoteInfo\x12O\n\tvalidator\x18\x01 \x01(\x0b\x32<.aea.valory.abci.v0_1_0.AbciMessage.LastCommitInfo.Validator\x12\x19\n\x11signed_last_block\x18\x02 \x01(\x08\x1a\x81\x01\n\x08ProofOps\x12\x41\n\x03ops\x18\x01 \x03(\x0b\x32\x34.aea.valory.abci.v0_1_0.AbciMessage.ProofOps.ProofOp\x1a\x32\n\x07ProofOp\x12\x0c\n\x04type\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\x0c\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\x0c\x1a\xb8\x01\n\x06Result\x12J\n\x0bresult_type\x18\x01 \x01(\x0e\x32\x35.aea.valory.abci.v0_1_0.AbciMessage.Result.ResultType"b\n\nResultType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\n\n\x06\x41\x43\x43\x45PT\x10\x01\x12\t\n\x05\x41\x42ORT\x10\x02\x12\n\n\x06REJECT\x10\x03\x12\x11\n\rREJECT_FORMAT\x10\x04\x12\x11\n\rREJECT_SENDER\x10\x05\x1aL\n\tSnapShots\x12?\n\tsnapshots\x18\x01 \x03(\x0b\x32,.aea.valory.abci.v0_1_0.AbciMessage.Snapshot\x1aZ\n\x08Snapshot\x12\x0e\n\x06height\x18\x01 \x01(\x04\x12\x0e\n\x06\x66ormat\x18\x02 \x01(\r\x12\x0e\n\x06\x63hunks\x18\x03 \x01(\r\x12\x0c\n\x04hash\x18\x04 \x01(\x0c\x12\x10\n\x08metadata\x18\x05 \x01(\x0c\x1a+\n\tTimestamp\x12\x0f\n\x07seconds\x18\x01 \x01(\x03\x12\r\n\x05nanos\x18\x02 \x01(\x05\x1a\x9b\x02\n\x10ValidatorUpdates\x12X\n\nvalidators\x18\x01 \x03(\x0b\x32\x44.aea.valory.abci.v0_1_0.AbciMessage.ValidatorUpdates.ValidatorUpdate\x1a:\n\tPublicKey\x12\x11\n\x07\x65\x64\x32\x35\x35\x31\x39\x18\x01 \x01(\x0cH\x00\x12\x13\n\tsecp256k1\x18\x02 \x01(\x0cH\x00\x42\x05\n\x03sum\x1aq\n\x0fValidatorUpdate\x12O\n\x07pub_key\x18\x01 \x01(\x0b\x32>.aea.valory.abci.v0_1_0.AbciMessage.ValidatorUpdates.PublicKey\x12\r\n\x05power\x18\x02 \x01(\x03\x1a,\n\x19Request_Echo_Performative\x12\x0f\n\x07message\x18\x01 \x01(\t\x1a\x1c\n\x1aRequest_Flush_Performative\x1aX\n\x19Request_Info_Performative\x12\x0f\n\x07version\x18\x01 \x01(\t\x12\x15\n\rblock_version\x18\x02 \x01(\x05\x12\x13\n\x0bp2p_version\x18\x03 \x01(\x05\x1aK\n\x1fRequest_Set_Option_Performative\x12\x12\n\noption_key\x18\x01 \x01(\t\x12\x14\n\x0coption_value\x18\x02 \x01(\t\x1a\xdb\x02\n\x1fRequest_Init_Chain_Performative\x12;\n\x04time\x18\x01 \x01(\x0b\x32-.aea.valory.abci.v0_1_0.AbciMessage.Timestamp\x12\x10\n\x08\x63hain_id\x18\x02 \x01(\t\x12M\n\x10\x63onsensus_params\x18\x03 \x01(\x0b\x32\x33.aea.valory.abci.v0_1_0.AbciMessage.ConsensusParams\x12\x1f\n\x17\x63onsensus_params_is_set\x18\x04 \x01(\x08\x12H\n\nvalidators\x18\x05 \x01(\x0b\x32\x34.aea.valory.abci.v0_1_0.AbciMessage.ValidatorUpdates\x12\x17\n\x0f\x61pp_state_bytes\x18\x06 \x01(\x0c\x12\x16\n\x0einitial_height\x18\x07 \x01(\x05\x1a]\n\x1aRequest_Query_Performative\x12\x12\n\nquery_data\x18\x01 \x01(\x0c\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x0e\n\x06height\x18\x03 \x01(\x05\x12\r\n\x05prove\x18\x04 \x01(\x08\x1a\x87\x02\n Request_Begin_Block_Performative\x12\x0c\n\x04hash\x18\x01 \x01(\x0c\x12:\n\x06header\x18\x02 \x01(\x0b\x32*.aea.valory.abci.v0_1_0.AbciMessage.Header\x12L\n\x10last_commit_info\x18\x03 \x01(\x0b\x32\x32.aea.valory.abci.v0_1_0.AbciMessage.LastCommitInfo\x12K\n\x14\x62yzantine_validators\x18\x04 \x01(\x0b\x32-.aea.valory.abci.v0_1_0.AbciMessage.Evidences\x1aj\n\x1dRequest_Check_Tx_Performative\x12\n\n\x02tx\x18\x01 \x01(\x0c\x12=\n\x04type\x18\x02 \x01(\x0b\x32/.aea.valory.abci.v0_1_0.AbciMessage.CheckTxType\x1a-\n\x1fRequest_Deliver_Tx_Performative\x12\n\n\x02tx\x18\x01 \x01(\x0c\x1a\x30\n\x1eRequest_End_Block_Performative\x12\x0e\n\x06height\x18\x01 \x01(\x05\x1a\x1d\n\x1bRequest_Commit_Performative\x1a%\n#Request_List_Snapshots_Performative\x1aw\n#Request_Offer_Snapshot_Performative\x12>\n\x08snapshot\x18\x01 \x01(\x0b\x32,.aea.valory.abci.v0_1_0.AbciMessage.Snapshot\x12\x10\n\x08\x61pp_hash\x18\x02 \x01(\x0c\x1a_\n(Request_Load_Snapshot_Chunk_Performative\x12\x0e\n\x06height\x18\x01 \x01(\x05\x12\x0e\n\x06\x66ormat\x18\x02 \x01(\x05\x12\x13\n\x0b\x63hunk_index\x18\x03 \x01(\x05\x1a_\n)Request_Apply_Snapshot_Chunk_Performative\x12\r\n\x05index\x18\x01 \x01(\x05\x12\r\n\x05\x63hunk\x18\x02 \x01(\x0c\x12\x14\n\x0c\x63hunk_sender\x18\x03 \x01(\t\x1a\x30\n\x1fResponse_Exception_Performative\x12\r\n\x05\x65rror\x18\x01 \x01(\t\x1a-\n\x1aResponse_Echo_Performative\x12\x0f\n\x07message\x18\x01 \x01(\t\x1a\x1d\n\x1bResponse_Flush_Performative\x1a\x8d\x01\n\x1aResponse_Info_Performative\x12\x11\n\tinfo_data\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12\x13\n\x0b\x61pp_version\x18\x03 \x01(\x05\x12\x19\n\x11last_block_height\x18\x04 \x01(\x05\x12\x1b\n\x13last_block_app_hash\x18\x05 \x01(\x0c\x1aK\n Response_Set_Option_Performative\x12\x0c\n\x04\x63ode\x18\x01 \x01(\x05\x12\x0b\n\x03log\x18\x02 \x01(\t\x12\x0c\n\x04info\x18\x03 \x01(\t\x1a\xee\x01\n Response_Init_Chain_Performative\x12M\n\x10\x63onsensus_params\x18\x01 \x01(\x0b\x32\x33.aea.valory.abci.v0_1_0.AbciMessage.ConsensusParams\x12\x1f\n\x17\x63onsensus_params_is_set\x18\x02 \x01(\x08\x12H\n\nvalidators\x18\x03 \x01(\x0b\x32\x34.aea.valory.abci.v0_1_0.AbciMessage.ValidatorUpdates\x12\x10\n\x08\x61pp_hash\x18\x04 \x01(\x0c\x1a\xd5\x01\n\x1bResponse_Query_Performative\x12\x0c\n\x04\x63ode\x18\x01 \x01(\x05\x12\x0b\n\x03log\x18\x02 \x01(\t\x12\x0c\n\x04info\x18\x03 \x01(\t\x12\r\n\x05index\x18\x04 \x01(\x05\x12\x0b\n\x03key\x18\x05 \x01(\x0c\x12\r\n\x05value\x18\x06 \x01(\x0c\x12?\n\tproof_ops\x18\x07 \x01(\x0b\x32,.aea.valory.abci.v0_1_0.AbciMessage.ProofOps\x12\x0e\n\x06height\x18\x08 \x01(\x05\x12\x11\n\tcodespace\x18\t \x01(\t\x1a_\n!Response_Begin_Block_Performative\x12:\n\x06\x65vents\x18\x01 \x01(\x0b\x32*.aea.valory.abci.v0_1_0.AbciMessage.Events\x1a\xcc\x01\n\x1eResponse_Check_Tx_Performative\x12\x0c\n\x04\x63ode\x18\x01 \x01(\x05\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\x12\x0b\n\x03log\x18\x03 \x01(\t\x12\x0c\n\x04info\x18\x04 \x01(\t\x12\x12\n\ngas_wanted\x18\x05 \x01(\x05\x12\x10\n\x08gas_used\x18\x06 \x01(\x05\x12:\n\x06\x65vents\x18\x07 \x01(\x0b\x32*.aea.valory.abci.v0_1_0.AbciMessage.Events\x12\x11\n\tcodespace\x18\x08 \x01(\t\x1a\xce\x01\n Response_Deliver_Tx_Performative\x12\x0c\n\x04\x63ode\x18\x01 \x01(\x05\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\x12\x0b\n\x03log\x18\x03 \x01(\t\x12\x0c\n\x04info\x18\x04 \x01(\t\x12\x12\n\ngas_wanted\x18\x05 \x01(\x05\x12\x10\n\x08gas_used\x18\x06 \x01(\x05\x12:\n\x06\x65vents\x18\x07 \x01(\x0b\x32*.aea.valory.abci.v0_1_0.AbciMessage.Events\x12\x11\n\tcodespace\x18\x08 \x01(\t\x1a\xac\x02\n\x1fResponse_End_Block_Performative\x12O\n\x11validator_updates\x18\x01 \x01(\x0b\x32\x34.aea.valory.abci.v0_1_0.AbciMessage.ValidatorUpdates\x12T\n\x17\x63onsensus_param_updates\x18\x02 \x01(\x0b\x32\x33.aea.valory.abci.v0_1_0.AbciMessage.ConsensusParams\x12&\n\x1e\x63onsensus_param_updates_is_set\x18\x03 \x01(\x08\x12:\n\x06\x65vents\x18\x04 \x01(\x0b\x32*.aea.valory.abci.v0_1_0.AbciMessage.Events\x1a\x43\n\x1cResponse_Commit_Performative\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\x12\x15\n\rretain_height\x18\x02 \x01(\x05\x1ah\n$Response_List_Snapshots_Performative\x12@\n\tsnapshots\x18\x01 \x01(\x0b\x32-.aea.valory.abci.v0_1_0.AbciMessage.SnapShots\x1a\x62\n$Response_Offer_Snapshot_Performative\x12:\n\x06result\x18\x01 \x01(\x0b\x32*.aea.valory.abci.v0_1_0.AbciMessage.Result\x1a:\n)Response_Load_Snapshot_Chunk_Performative\x12\r\n\x05\x63hunk\x18\x01 \x01(\x0c\x1a\x98\x01\n*Response_Apply_Snapshot_Chunk_Performative\x12:\n\x06result\x18\x01 \x01(\x0b\x32*.aea.valory.abci.v0_1_0.AbciMessage.Result\x12\x16\n\x0erefetch_chunks\x18\x02 \x03(\x05\x12\x16\n\x0ereject_senders\x18\x03 \x03(\t\x1ai\n\x12\x44ummy_Performative\x12S\n\x16\x64ummy_consensus_params\x18\x01 \x01(\x0b\x32\x33.aea.valory.abci.v0_1_0.AbciMessage.ConsensusParamsB\x0e\n\x0cperformativeb\x06proto3'
)


_ABCIMESSAGE = DESCRIPTOR.message_types_by_name["AbciMessage"]
_ABCIMESSAGE_CHECKTXTYPE = _ABCIMESSAGE.nested_types_by_name["CheckTxType"]
_ABCIMESSAGE_CONSENSUSPARAMS = _ABCIMESSAGE.nested_types_by_name["ConsensusParams"]
_ABCIMESSAGE_CONSENSUSPARAMS_DURATION = (
    _ABCIMESSAGE_CONSENSUSPARAMS.nested_types_by_name["Duration"]
)
_ABCIMESSAGE_CONSENSUSPARAMS_BLOCKPARAMS = (
    _ABCIMESSAGE_CONSENSUSPARAMS.nested_types_by_name["BlockParams"]
)
_ABCIMESSAGE_CONSENSUSPARAMS_EVIDENCEPARAMS = (
    _ABCIMESSAGE_CONSENSUSPARAMS.nested_types_by_name["EvidenceParams"]
)
_ABCIMESSAGE_CONSENSUSPARAMS_VALIDATORPARAMS = (
    _ABCIMESSAGE_CONSENSUSPARAMS.nested_types_by_name["ValidatorParams"]
)
_ABCIMESSAGE_CONSENSUSPARAMS_VERSIONPARAMS = (
    _ABCIMESSAGE_CONSENSUSPARAMS.nested_types_by_name["VersionParams"]
)
_ABCIMESSAGE_EVENTS = _ABCIMESSAGE.nested_types_by_name["Events"]
_ABCIMESSAGE_EVENTS_EVENTATTRIBUTE = _ABCIMESSAGE_EVENTS.nested_types_by_name[
    "EventAttribute"
]
_ABCIMESSAGE_EVENTS_EVENT = _ABCIMESSAGE_EVENTS.nested_types_by_name["Event"]
_ABCIMESSAGE_EVIDENCES = _ABCIMESSAGE.nested_types_by_name["Evidences"]
_ABCIMESSAGE_EVIDENCES_EVIDENCE = _ABCIMESSAGE_EVIDENCES.nested_types_by_name[
    "Evidence"
]
_ABCIMESSAGE_HEADER = _ABCIMESSAGE.nested_types_by_name["Header"]
_ABCIMESSAGE_HEADER_CONSENSUSVERSION = _ABCIMESSAGE_HEADER.nested_types_by_name[
    "ConsensusVersion"
]
_ABCIMESSAGE_HEADER_BLOCKID = _ABCIMESSAGE_HEADER.nested_types_by_name["BlockID"]
_ABCIMESSAGE_HEADER_PARTSETHEADER = _ABCIMESSAGE_HEADER.nested_types_by_name[
    "PartSetHeader"
]
_ABCIMESSAGE_LASTCOMMITINFO = _ABCIMESSAGE.nested_types_by_name["LastCommitInfo"]
_ABCIMESSAGE_LASTCOMMITINFO_VALIDATOR = (
    _ABCIMESSAGE_LASTCOMMITINFO.nested_types_by_name["Validator"]
)
_ABCIMESSAGE_LASTCOMMITINFO_VOTEINFO = _ABCIMESSAGE_LASTCOMMITINFO.nested_types_by_name[
    "VoteInfo"
]
_ABCIMESSAGE_PROOFOPS = _ABCIMESSAGE.nested_types_by_name["ProofOps"]
_ABCIMESSAGE_PROOFOPS_PROOFOP = _ABCIMESSAGE_PROOFOPS.nested_types_by_name["ProofOp"]
_ABCIMESSAGE_RESULT = _ABCIMESSAGE.nested_types_by_name["Result"]
_ABCIMESSAGE_SNAPSHOTS = _ABCIMESSAGE.nested_types_by_name["SnapShots"]
_ABCIMESSAGE_SNAPSHOT = _ABCIMESSAGE.nested_types_by_name["Snapshot"]
_ABCIMESSAGE_TIMESTAMP = _ABCIMESSAGE.nested_types_by_name["Timestamp"]
_ABCIMESSAGE_VALIDATORUPDATES = _ABCIMESSAGE.nested_types_by_name["ValidatorUpdates"]
_ABCIMESSAGE_VALIDATORUPDATES_PUBLICKEY = (
    _ABCIMESSAGE_VALIDATORUPDATES.nested_types_by_name["PublicKey"]
)
_ABCIMESSAGE_VALIDATORUPDATES_VALIDATORUPDATE = (
    _ABCIMESSAGE_VALIDATORUPDATES.nested_types_by_name["ValidatorUpdate"]
)
_ABCIMESSAGE_REQUEST_ECHO_PERFORMATIVE = _ABCIMESSAGE.nested_types_by_name[
    "Request_Echo_Performative"
]
_ABCIMESSAGE_REQUEST_FLUSH_PERFORMATIVE = _ABCIMESSAGE.nested_types_by_name[
    "Request_Flush_Performative"
]
_ABCIMESSAGE_REQUEST_INFO_PERFORMATIVE = _ABCIMESSAGE.nested_types_by_name[
    "Request_Info_Performative"
]
_ABCIMESSAGE_REQUEST_SET_OPTION_PERFORMATIVE = _ABCIMESSAGE.nested_types_by_name[
    "Request_Set_Option_Performative"
]
_ABCIMESSAGE_REQUEST_INIT_CHAIN_PERFORMATIVE = _ABCIMESSAGE.nested_types_by_name[
    "Request_Init_Chain_Performative"
]
_ABCIMESSAGE_REQUEST_QUERY_PERFORMATIVE = _ABCIMESSAGE.nested_types_by_name[
    "Request_Query_Performative"
]
_ABCIMESSAGE_REQUEST_BEGIN_BLOCK_PERFORMATIVE = _ABCIMESSAGE.nested_types_by_name[
    "Request_Begin_Block_Performative"
]
_ABCIMESSAGE_REQUEST_CHECK_TX_PERFORMATIVE = _ABCIMESSAGE.nested_types_by_name[
    "Request_Check_Tx_Performative"
]
_ABCIMESSAGE_REQUEST_DELIVER_TX_PERFORMATIVE = _ABCIMESSAGE.nested_types_by_name[
    "Request_Deliver_Tx_Performative"
]
_ABCIMESSAGE_REQUEST_END_BLOCK_PERFORMATIVE = _ABCIMESSAGE.nested_types_by_name[
    "Request_End_Block_Performative"
]
_ABCIMESSAGE_REQUEST_COMMIT_PERFORMATIVE = _ABCIMESSAGE.nested_types_by_name[
    "Request_Commit_Performative"
]
_ABCIMESSAGE_REQUEST_LIST_SNAPSHOTS_PERFORMATIVE = _ABCIMESSAGE.nested_types_by_name[
    "Request_List_Snapshots_Performative"
]
_ABCIMESSAGE_REQUEST_OFFER_SNAPSHOT_PERFORMATIVE = _ABCIMESSAGE.nested_types_by_name[
    "Request_Offer_Snapshot_Performative"
]
_ABCIMESSAGE_REQUEST_LOAD_SNAPSHOT_CHUNK_PERFORMATIVE = (
    _ABCIMESSAGE.nested_types_by_name["Request_Load_Snapshot_Chunk_Performative"]
)
_ABCIMESSAGE_REQUEST_APPLY_SNAPSHOT_CHUNK_PERFORMATIVE = (
    _ABCIMESSAGE.nested_types_by_name["Request_Apply_Snapshot_Chunk_Performative"]
)
_ABCIMESSAGE_RESPONSE_EXCEPTION_PERFORMATIVE = _ABCIMESSAGE.nested_types_by_name[
    "Response_Exception_Performative"
]
_ABCIMESSAGE_RESPONSE_ECHO_PERFORMATIVE = _ABCIMESSAGE.nested_types_by_name[
    "Response_Echo_Performative"
]
_ABCIMESSAGE_RESPONSE_FLUSH_PERFORMATIVE = _ABCIMESSAGE.nested_types_by_name[
    "Response_Flush_Performative"
]
_ABCIMESSAGE_RESPONSE_INFO_PERFORMATIVE = _ABCIMESSAGE.nested_types_by_name[
    "Response_Info_Performative"
]
_ABCIMESSAGE_RESPONSE_SET_OPTION_PERFORMATIVE = _ABCIMESSAGE.nested_types_by_name[
    "Response_Set_Option_Performative"
]
_ABCIMESSAGE_RESPONSE_INIT_CHAIN_PERFORMATIVE = _ABCIMESSAGE.nested_types_by_name[
    "Response_Init_Chain_Performative"
]
_ABCIMESSAGE_RESPONSE_QUERY_PERFORMATIVE = _ABCIMESSAGE.nested_types_by_name[
    "Response_Query_Performative"
]
_ABCIMESSAGE_RESPONSE_BEGIN_BLOCK_PERFORMATIVE = _ABCIMESSAGE.nested_types_by_name[
    "Response_Begin_Block_Performative"
]
_ABCIMESSAGE_RESPONSE_CHECK_TX_PERFORMATIVE = _ABCIMESSAGE.nested_types_by_name[
    "Response_Check_Tx_Performative"
]
_ABCIMESSAGE_RESPONSE_DELIVER_TX_PERFORMATIVE = _ABCIMESSAGE.nested_types_by_name[
    "Response_Deliver_Tx_Performative"
]
_ABCIMESSAGE_RESPONSE_END_BLOCK_PERFORMATIVE = _ABCIMESSAGE.nested_types_by_name[
    "Response_End_Block_Performative"
]
_ABCIMESSAGE_RESPONSE_COMMIT_PERFORMATIVE = _ABCIMESSAGE.nested_types_by_name[
    "Response_Commit_Performative"
]
_ABCIMESSAGE_RESPONSE_LIST_SNAPSHOTS_PERFORMATIVE = _ABCIMESSAGE.nested_types_by_name[
    "Response_List_Snapshots_Performative"
]
_ABCIMESSAGE_RESPONSE_OFFER_SNAPSHOT_PERFORMATIVE = _ABCIMESSAGE.nested_types_by_name[
    "Response_Offer_Snapshot_Performative"
]
_ABCIMESSAGE_RESPONSE_LOAD_SNAPSHOT_CHUNK_PERFORMATIVE = (
    _ABCIMESSAGE.nested_types_by_name["Response_Load_Snapshot_Chunk_Performative"]
)
_ABCIMESSAGE_RESPONSE_APPLY_SNAPSHOT_CHUNK_PERFORMATIVE = (
    _ABCIMESSAGE.nested_types_by_name["Response_Apply_Snapshot_Chunk_Performative"]
)
_ABCIMESSAGE_DUMMY_PERFORMATIVE = _ABCIMESSAGE.nested_types_by_name[
    "Dummy_Performative"
]
_ABCIMESSAGE_CHECKTXTYPE__CHECKTXTYPE = _ABCIMESSAGE_CHECKTXTYPE.enum_types_by_name[
    "_CheckTxType"
]
_ABCIMESSAGE_EVIDENCES_EVIDENCE_EVIDENCETYPE = (
    _ABCIMESSAGE_EVIDENCES_EVIDENCE.enum_types_by_name["EvidenceType"]
)
_ABCIMESSAGE_RESULT_RESULTTYPE = _ABCIMESSAGE_RESULT.enum_types_by_name["ResultType"]
AbciMessage = _reflection.GeneratedProtocolMessageType(
    "AbciMessage",
    (_message.Message,),
    {
        "CheckTxType": _reflection.GeneratedProtocolMessageType(
            "CheckTxType",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_CHECKTXTYPE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.CheckTxType)
            },
        ),
        "ConsensusParams": _reflection.GeneratedProtocolMessageType(
            "ConsensusParams",
            (_message.Message,),
            {
                "Duration": _reflection.GeneratedProtocolMessageType(
                    "Duration",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _ABCIMESSAGE_CONSENSUSPARAMS_DURATION,
                        "__module__": "abci_pb2"
                        # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.ConsensusParams.Duration)
                    },
                ),
                "BlockParams": _reflection.GeneratedProtocolMessageType(
                    "BlockParams",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _ABCIMESSAGE_CONSENSUSPARAMS_BLOCKPARAMS,
                        "__module__": "abci_pb2"
                        # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.ConsensusParams.BlockParams)
                    },
                ),
                "EvidenceParams": _reflection.GeneratedProtocolMessageType(
                    "EvidenceParams",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _ABCIMESSAGE_CONSENSUSPARAMS_EVIDENCEPARAMS,
                        "__module__": "abci_pb2"
                        # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.ConsensusParams.EvidenceParams)
                    },
                ),
                "ValidatorParams": _reflection.GeneratedProtocolMessageType(
                    "ValidatorParams",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _ABCIMESSAGE_CONSENSUSPARAMS_VALIDATORPARAMS,
                        "__module__": "abci_pb2"
                        # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.ConsensusParams.ValidatorParams)
                    },
                ),
                "VersionParams": _reflection.GeneratedProtocolMessageType(
                    "VersionParams",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _ABCIMESSAGE_CONSENSUSPARAMS_VERSIONPARAMS,
                        "__module__": "abci_pb2"
                        # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.ConsensusParams.VersionParams)
                    },
                ),
                "DESCRIPTOR": _ABCIMESSAGE_CONSENSUSPARAMS,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.ConsensusParams)
            },
        ),
        "Events": _reflection.GeneratedProtocolMessageType(
            "Events",
            (_message.Message,),
            {
                "EventAttribute": _reflection.GeneratedProtocolMessageType(
                    "EventAttribute",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _ABCIMESSAGE_EVENTS_EVENTATTRIBUTE,
                        "__module__": "abci_pb2"
                        # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Events.EventAttribute)
                    },
                ),
                "Event": _reflection.GeneratedProtocolMessageType(
                    "Event",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _ABCIMESSAGE_EVENTS_EVENT,
                        "__module__": "abci_pb2"
                        # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Events.Event)
                    },
                ),
                "DESCRIPTOR": _ABCIMESSAGE_EVENTS,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Events)
            },
        ),
        "Evidences": _reflection.GeneratedProtocolMessageType(
            "Evidences",
            (_message.Message,),
            {
                "Evidence": _reflection.GeneratedProtocolMessageType(
                    "Evidence",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _ABCIMESSAGE_EVIDENCES_EVIDENCE,
                        "__module__": "abci_pb2"
                        # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Evidences.Evidence)
                    },
                ),
                "DESCRIPTOR": _ABCIMESSAGE_EVIDENCES,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Evidences)
            },
        ),
        "Header": _reflection.GeneratedProtocolMessageType(
            "Header",
            (_message.Message,),
            {
                "ConsensusVersion": _reflection.GeneratedProtocolMessageType(
                    "ConsensusVersion",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _ABCIMESSAGE_HEADER_CONSENSUSVERSION,
                        "__module__": "abci_pb2"
                        # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Header.ConsensusVersion)
                    },
                ),
                "BlockID": _reflection.GeneratedProtocolMessageType(
                    "BlockID",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _ABCIMESSAGE_HEADER_BLOCKID,
                        "__module__": "abci_pb2"
                        # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Header.BlockID)
                    },
                ),
                "PartSetHeader": _reflection.GeneratedProtocolMessageType(
                    "PartSetHeader",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _ABCIMESSAGE_HEADER_PARTSETHEADER,
                        "__module__": "abci_pb2"
                        # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Header.PartSetHeader)
                    },
                ),
                "DESCRIPTOR": _ABCIMESSAGE_HEADER,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Header)
            },
        ),
        "LastCommitInfo": _reflection.GeneratedProtocolMessageType(
            "LastCommitInfo",
            (_message.Message,),
            {
                "Validator": _reflection.GeneratedProtocolMessageType(
                    "Validator",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _ABCIMESSAGE_LASTCOMMITINFO_VALIDATOR,
                        "__module__": "abci_pb2"
                        # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.LastCommitInfo.Validator)
                    },
                ),
                "VoteInfo": _reflection.GeneratedProtocolMessageType(
                    "VoteInfo",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _ABCIMESSAGE_LASTCOMMITINFO_VOTEINFO,
                        "__module__": "abci_pb2"
                        # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.LastCommitInfo.VoteInfo)
                    },
                ),
                "DESCRIPTOR": _ABCIMESSAGE_LASTCOMMITINFO,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.LastCommitInfo)
            },
        ),
        "ProofOps": _reflection.GeneratedProtocolMessageType(
            "ProofOps",
            (_message.Message,),
            {
                "ProofOp": _reflection.GeneratedProtocolMessageType(
                    "ProofOp",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _ABCIMESSAGE_PROOFOPS_PROOFOP,
                        "__module__": "abci_pb2"
                        # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.ProofOps.ProofOp)
                    },
                ),
                "DESCRIPTOR": _ABCIMESSAGE_PROOFOPS,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.ProofOps)
            },
        ),
        "Result": _reflection.GeneratedProtocolMessageType(
            "Result",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_RESULT,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Result)
            },
        ),
        "SnapShots": _reflection.GeneratedProtocolMessageType(
            "SnapShots",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_SNAPSHOTS,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.SnapShots)
            },
        ),
        "Snapshot": _reflection.GeneratedProtocolMessageType(
            "Snapshot",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_SNAPSHOT,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Snapshot)
            },
        ),
        "Timestamp": _reflection.GeneratedProtocolMessageType(
            "Timestamp",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_TIMESTAMP,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Timestamp)
            },
        ),
        "ValidatorUpdates": _reflection.GeneratedProtocolMessageType(
            "ValidatorUpdates",
            (_message.Message,),
            {
                "PublicKey": _reflection.GeneratedProtocolMessageType(
                    "PublicKey",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _ABCIMESSAGE_VALIDATORUPDATES_PUBLICKEY,
                        "__module__": "abci_pb2"
                        # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.ValidatorUpdates.PublicKey)
                    },
                ),
                "ValidatorUpdate": _reflection.GeneratedProtocolMessageType(
                    "ValidatorUpdate",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _ABCIMESSAGE_VALIDATORUPDATES_VALIDATORUPDATE,
                        "__module__": "abci_pb2"
                        # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.ValidatorUpdates.ValidatorUpdate)
                    },
                ),
                "DESCRIPTOR": _ABCIMESSAGE_VALIDATORUPDATES,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.ValidatorUpdates)
            },
        ),
        "Request_Echo_Performative": _reflection.GeneratedProtocolMessageType(
            "Request_Echo_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_REQUEST_ECHO_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Request_Echo_Performative)
            },
        ),
        "Request_Flush_Performative": _reflection.GeneratedProtocolMessageType(
            "Request_Flush_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_REQUEST_FLUSH_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Request_Flush_Performative)
            },
        ),
        "Request_Info_Performative": _reflection.GeneratedProtocolMessageType(
            "Request_Info_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_REQUEST_INFO_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Request_Info_Performative)
            },
        ),
        "Request_Set_Option_Performative": _reflection.GeneratedProtocolMessageType(
            "Request_Set_Option_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_REQUEST_SET_OPTION_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Request_Set_Option_Performative)
            },
        ),
        "Request_Init_Chain_Performative": _reflection.GeneratedProtocolMessageType(
            "Request_Init_Chain_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_REQUEST_INIT_CHAIN_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Request_Init_Chain_Performative)
            },
        ),
        "Request_Query_Performative": _reflection.GeneratedProtocolMessageType(
            "Request_Query_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_REQUEST_QUERY_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Request_Query_Performative)
            },
        ),
        "Request_Begin_Block_Performative": _reflection.GeneratedProtocolMessageType(
            "Request_Begin_Block_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_REQUEST_BEGIN_BLOCK_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Request_Begin_Block_Performative)
            },
        ),
        "Request_Check_Tx_Performative": _reflection.GeneratedProtocolMessageType(
            "Request_Check_Tx_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_REQUEST_CHECK_TX_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Request_Check_Tx_Performative)
            },
        ),
        "Request_Deliver_Tx_Performative": _reflection.GeneratedProtocolMessageType(
            "Request_Deliver_Tx_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_REQUEST_DELIVER_TX_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Request_Deliver_Tx_Performative)
            },
        ),
        "Request_End_Block_Performative": _reflection.GeneratedProtocolMessageType(
            "Request_End_Block_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_REQUEST_END_BLOCK_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Request_End_Block_Performative)
            },
        ),
        "Request_Commit_Performative": _reflection.GeneratedProtocolMessageType(
            "Request_Commit_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_REQUEST_COMMIT_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Request_Commit_Performative)
            },
        ),
        "Request_List_Snapshots_Performative": _reflection.GeneratedProtocolMessageType(
            "Request_List_Snapshots_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_REQUEST_LIST_SNAPSHOTS_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Request_List_Snapshots_Performative)
            },
        ),
        "Request_Offer_Snapshot_Performative": _reflection.GeneratedProtocolMessageType(
            "Request_Offer_Snapshot_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_REQUEST_OFFER_SNAPSHOT_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Request_Offer_Snapshot_Performative)
            },
        ),
        "Request_Load_Snapshot_Chunk_Performative": _reflection.GeneratedProtocolMessageType(
            "Request_Load_Snapshot_Chunk_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_REQUEST_LOAD_SNAPSHOT_CHUNK_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Request_Load_Snapshot_Chunk_Performative)
            },
        ),
        "Request_Apply_Snapshot_Chunk_Performative": _reflection.GeneratedProtocolMessageType(
            "Request_Apply_Snapshot_Chunk_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_REQUEST_APPLY_SNAPSHOT_CHUNK_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Request_Apply_Snapshot_Chunk_Performative)
            },
        ),
        "Response_Exception_Performative": _reflection.GeneratedProtocolMessageType(
            "Response_Exception_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_RESPONSE_EXCEPTION_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Response_Exception_Performative)
            },
        ),
        "Response_Echo_Performative": _reflection.GeneratedProtocolMessageType(
            "Response_Echo_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_RESPONSE_ECHO_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Response_Echo_Performative)
            },
        ),
        "Response_Flush_Performative": _reflection.GeneratedProtocolMessageType(
            "Response_Flush_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_RESPONSE_FLUSH_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Response_Flush_Performative)
            },
        ),
        "Response_Info_Performative": _reflection.GeneratedProtocolMessageType(
            "Response_Info_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_RESPONSE_INFO_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Response_Info_Performative)
            },
        ),
        "Response_Set_Option_Performative": _reflection.GeneratedProtocolMessageType(
            "Response_Set_Option_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_RESPONSE_SET_OPTION_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Response_Set_Option_Performative)
            },
        ),
        "Response_Init_Chain_Performative": _reflection.GeneratedProtocolMessageType(
            "Response_Init_Chain_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_RESPONSE_INIT_CHAIN_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Response_Init_Chain_Performative)
            },
        ),
        "Response_Query_Performative": _reflection.GeneratedProtocolMessageType(
            "Response_Query_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_RESPONSE_QUERY_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Response_Query_Performative)
            },
        ),
        "Response_Begin_Block_Performative": _reflection.GeneratedProtocolMessageType(
            "Response_Begin_Block_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_RESPONSE_BEGIN_BLOCK_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Response_Begin_Block_Performative)
            },
        ),
        "Response_Check_Tx_Performative": _reflection.GeneratedProtocolMessageType(
            "Response_Check_Tx_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_RESPONSE_CHECK_TX_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Response_Check_Tx_Performative)
            },
        ),
        "Response_Deliver_Tx_Performative": _reflection.GeneratedProtocolMessageType(
            "Response_Deliver_Tx_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_RESPONSE_DELIVER_TX_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Response_Deliver_Tx_Performative)
            },
        ),
        "Response_End_Block_Performative": _reflection.GeneratedProtocolMessageType(
            "Response_End_Block_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_RESPONSE_END_BLOCK_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Response_End_Block_Performative)
            },
        ),
        "Response_Commit_Performative": _reflection.GeneratedProtocolMessageType(
            "Response_Commit_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_RESPONSE_COMMIT_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Response_Commit_Performative)
            },
        ),
        "Response_List_Snapshots_Performative": _reflection.GeneratedProtocolMessageType(
            "Response_List_Snapshots_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_RESPONSE_LIST_SNAPSHOTS_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Response_List_Snapshots_Performative)
            },
        ),
        "Response_Offer_Snapshot_Performative": _reflection.GeneratedProtocolMessageType(
            "Response_Offer_Snapshot_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_RESPONSE_OFFER_SNAPSHOT_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Response_Offer_Snapshot_Performative)
            },
        ),
        "Response_Load_Snapshot_Chunk_Performative": _reflection.GeneratedProtocolMessageType(
            "Response_Load_Snapshot_Chunk_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_RESPONSE_LOAD_SNAPSHOT_CHUNK_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Response_Load_Snapshot_Chunk_Performative)
            },
        ),
        "Response_Apply_Snapshot_Chunk_Performative": _reflection.GeneratedProtocolMessageType(
            "Response_Apply_Snapshot_Chunk_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_RESPONSE_APPLY_SNAPSHOT_CHUNK_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Response_Apply_Snapshot_Chunk_Performative)
            },
        ),
        "Dummy_Performative": _reflection.GeneratedProtocolMessageType(
            "Dummy_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ABCIMESSAGE_DUMMY_PERFORMATIVE,
                "__module__": "abci_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage.Dummy_Performative)
            },
        ),
        "DESCRIPTOR": _ABCIMESSAGE,
        "__module__": "abci_pb2"
        # @@protoc_insertion_point(class_scope:aea.valory.abci.v0_1_0.AbciMessage)
    },
)
_sym_db.RegisterMessage(AbciMessage)
_sym_db.RegisterMessage(AbciMessage.CheckTxType)
_sym_db.RegisterMessage(AbciMessage.ConsensusParams)
_sym_db.RegisterMessage(AbciMessage.ConsensusParams.Duration)
_sym_db.RegisterMessage(AbciMessage.ConsensusParams.BlockParams)
_sym_db.RegisterMessage(AbciMessage.ConsensusParams.EvidenceParams)
_sym_db.RegisterMessage(AbciMessage.ConsensusParams.ValidatorParams)
_sym_db.RegisterMessage(AbciMessage.ConsensusParams.VersionParams)
_sym_db.RegisterMessage(AbciMessage.Events)
_sym_db.RegisterMessage(AbciMessage.Events.EventAttribute)
_sym_db.RegisterMessage(AbciMessage.Events.Event)
_sym_db.RegisterMessage(AbciMessage.Evidences)
_sym_db.RegisterMessage(AbciMessage.Evidences.Evidence)
_sym_db.RegisterMessage(AbciMessage.Header)
_sym_db.RegisterMessage(AbciMessage.Header.ConsensusVersion)
_sym_db.RegisterMessage(AbciMessage.Header.BlockID)
_sym_db.RegisterMessage(AbciMessage.Header.PartSetHeader)
_sym_db.RegisterMessage(AbciMessage.LastCommitInfo)
_sym_db.RegisterMessage(AbciMessage.LastCommitInfo.Validator)
_sym_db.RegisterMessage(AbciMessage.LastCommitInfo.VoteInfo)
_sym_db.RegisterMessage(AbciMessage.ProofOps)
_sym_db.RegisterMessage(AbciMessage.ProofOps.ProofOp)
_sym_db.RegisterMessage(AbciMessage.Result)
_sym_db.RegisterMessage(AbciMessage.SnapShots)
_sym_db.RegisterMessage(AbciMessage.Snapshot)
_sym_db.RegisterMessage(AbciMessage.Timestamp)
_sym_db.RegisterMessage(AbciMessage.ValidatorUpdates)
_sym_db.RegisterMessage(AbciMessage.ValidatorUpdates.PublicKey)
_sym_db.RegisterMessage(AbciMessage.ValidatorUpdates.ValidatorUpdate)
_sym_db.RegisterMessage(AbciMessage.Request_Echo_Performative)
_sym_db.RegisterMessage(AbciMessage.Request_Flush_Performative)
_sym_db.RegisterMessage(AbciMessage.Request_Info_Performative)
_sym_db.RegisterMessage(AbciMessage.Request_Set_Option_Performative)
_sym_db.RegisterMessage(AbciMessage.Request_Init_Chain_Performative)
_sym_db.RegisterMessage(AbciMessage.Request_Query_Performative)
_sym_db.RegisterMessage(AbciMessage.Request_Begin_Block_Performative)
_sym_db.RegisterMessage(AbciMessage.Request_Check_Tx_Performative)
_sym_db.RegisterMessage(AbciMessage.Request_Deliver_Tx_Performative)
_sym_db.RegisterMessage(AbciMessage.Request_End_Block_Performative)
_sym_db.RegisterMessage(AbciMessage.Request_Commit_Performative)
_sym_db.RegisterMessage(AbciMessage.Request_List_Snapshots_Performative)
_sym_db.RegisterMessage(AbciMessage.Request_Offer_Snapshot_Performative)
_sym_db.RegisterMessage(AbciMessage.Request_Load_Snapshot_Chunk_Performative)
_sym_db.RegisterMessage(AbciMessage.Request_Apply_Snapshot_Chunk_Performative)
_sym_db.RegisterMessage(AbciMessage.Response_Exception_Performative)
_sym_db.RegisterMessage(AbciMessage.Response_Echo_Performative)
_sym_db.RegisterMessage(AbciMessage.Response_Flush_Performative)
_sym_db.RegisterMessage(AbciMessage.Response_Info_Performative)
_sym_db.RegisterMessage(AbciMessage.Response_Set_Option_Performative)
_sym_db.RegisterMessage(AbciMessage.Response_Init_Chain_Performative)
_sym_db.RegisterMessage(AbciMessage.Response_Query_Performative)
_sym_db.RegisterMessage(AbciMessage.Response_Begin_Block_Performative)
_sym_db.RegisterMessage(AbciMessage.Response_Check_Tx_Performative)
_sym_db.RegisterMessage(AbciMessage.Response_Deliver_Tx_Performative)
_sym_db.RegisterMessage(AbciMessage.Response_End_Block_Performative)
_sym_db.RegisterMessage(AbciMessage.Response_Commit_Performative)
_sym_db.RegisterMessage(AbciMessage.Response_List_Snapshots_Performative)
_sym_db.RegisterMessage(AbciMessage.Response_Offer_Snapshot_Performative)
_sym_db.RegisterMessage(AbciMessage.Response_Load_Snapshot_Chunk_Performative)
_sym_db.RegisterMessage(AbciMessage.Response_Apply_Snapshot_Chunk_Performative)
_sym_db.RegisterMessage(AbciMessage.Dummy_Performative)

if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    _ABCIMESSAGE._serialized_start = 39
    _ABCIMESSAGE._serialized_end = 10285
    _ABCIMESSAGE_CHECKTXTYPE._serialized_start = 3222
    _ABCIMESSAGE_CHECKTXTYPE._serialized_end = 3349
    _ABCIMESSAGE_CHECKTXTYPE__CHECKTXTYPE._serialized_start = 3313
    _ABCIMESSAGE_CHECKTXTYPE__CHECKTXTYPE._serialized_end = 3349
    _ABCIMESSAGE_CONSENSUSPARAMS._serialized_start = 3352
    _ABCIMESSAGE_CONSENSUSPARAMS._serialized_end = 4036
    _ABCIMESSAGE_CONSENSUSPARAMS_DURATION._serialized_start = 3709
    _ABCIMESSAGE_CONSENSUSPARAMS_DURATION._serialized_end = 3751
    _ABCIMESSAGE_CONSENSUSPARAMS_BLOCKPARAMS._serialized_start = 3753
    _ABCIMESSAGE_CONSENSUSPARAMS_BLOCKPARAMS._serialized_end = 3802
    _ABCIMESSAGE_CONSENSUSPARAMS_EVIDENCEPARAMS._serialized_start = 3805
    _ABCIMESSAGE_CONSENSUSPARAMS_EVIDENCEPARAMS._serialized_end = 3956
    _ABCIMESSAGE_CONSENSUSPARAMS_VALIDATORPARAMS._serialized_start = 3958
    _ABCIMESSAGE_CONSENSUSPARAMS_VALIDATORPARAMS._serialized_end = 3998
    _ABCIMESSAGE_CONSENSUSPARAMS_VERSIONPARAMS._serialized_start = 4000
    _ABCIMESSAGE_CONSENSUSPARAMS_VERSIONPARAMS._serialized_end = 4036
    _ABCIMESSAGE_EVENTS._serialized_start = 4039
    _ABCIMESSAGE_EVENTS._serialized_end = 4276
    _ABCIMESSAGE_EVENTS_EVENTATTRIBUTE._serialized_start = 4115
    _ABCIMESSAGE_EVENTS_EVENTATTRIBUTE._serialized_end = 4174
    _ABCIMESSAGE_EVENTS_EVENT._serialized_start = 4176
    _ABCIMESSAGE_EVENTS_EVENT._serialized_end = 4276
    _ABCIMESSAGE_EVIDENCES._serialized_start = 4279
    _ABCIMESSAGE_EVIDENCES._serialized_end = 4732
    _ABCIMESSAGE_EVIDENCES_EVIDENCE._serialized_start = 4379
    _ABCIMESSAGE_EVIDENCES_EVIDENCE._serialized_end = 4732
    _ABCIMESSAGE_EVIDENCES_EVIDENCE_EVIDENCETYPE._serialized_start = 4660
    _ABCIMESSAGE_EVIDENCES_EVIDENCE_EVIDENCETYPE._serialized_end = 4732
    _ABCIMESSAGE_HEADER._serialized_start = 4735
    _ABCIMESSAGE_HEADER._serialized_end = 5411
    _ABCIMESSAGE_HEADER_CONSENSUSVERSION._serialized_start = 5211
    _ABCIMESSAGE_HEADER_CONSENSUSVERSION._serialized_end = 5257
    _ABCIMESSAGE_HEADER_BLOCKID._serialized_start = 5259
    _ABCIMESSAGE_HEADER_BLOCKID._serialized_end = 5365
    _ABCIMESSAGE_HEADER_PARTSETHEADER._serialized_start = 5367
    _ABCIMESSAGE_HEADER_PARTSETHEADER._serialized_end = 5411
    _ABCIMESSAGE_LASTCOMMITINFO._serialized_start = 5414
    _ABCIMESSAGE_LASTCOMMITINFO._serialized_end = 5686
    _ABCIMESSAGE_LASTCOMMITINFO_VALIDATOR._serialized_start = 5523
    _ABCIMESSAGE_LASTCOMMITINFO_VALIDATOR._serialized_end = 5566
    _ABCIMESSAGE_LASTCOMMITINFO_VOTEINFO._serialized_start = 5568
    _ABCIMESSAGE_LASTCOMMITINFO_VOTEINFO._serialized_end = 5686
    _ABCIMESSAGE_PROOFOPS._serialized_start = 5689
    _ABCIMESSAGE_PROOFOPS._serialized_end = 5818
    _ABCIMESSAGE_PROOFOPS_PROOFOP._serialized_start = 5768
    _ABCIMESSAGE_PROOFOPS_PROOFOP._serialized_end = 5818
    _ABCIMESSAGE_RESULT._serialized_start = 5821
    _ABCIMESSAGE_RESULT._serialized_end = 6005
    _ABCIMESSAGE_RESULT_RESULTTYPE._serialized_start = 5907
    _ABCIMESSAGE_RESULT_RESULTTYPE._serialized_end = 6005
    _ABCIMESSAGE_SNAPSHOTS._serialized_start = 6007
    _ABCIMESSAGE_SNAPSHOTS._serialized_end = 6083
    _ABCIMESSAGE_SNAPSHOT._serialized_start = 6085
    _ABCIMESSAGE_SNAPSHOT._serialized_end = 6175
    _ABCIMESSAGE_TIMESTAMP._serialized_start = 6177
    _ABCIMESSAGE_TIMESTAMP._serialized_end = 6220
    _ABCIMESSAGE_VALIDATORUPDATES._serialized_start = 6223
    _ABCIMESSAGE_VALIDATORUPDATES._serialized_end = 6506
    _ABCIMESSAGE_VALIDATORUPDATES_PUBLICKEY._serialized_start = 6333
    _ABCIMESSAGE_VALIDATORUPDATES_PUBLICKEY._serialized_end = 6391
    _ABCIMESSAGE_VALIDATORUPDATES_VALIDATORUPDATE._serialized_start = 6393
    _ABCIMESSAGE_VALIDATORUPDATES_VALIDATORUPDATE._serialized_end = 6506
    _ABCIMESSAGE_REQUEST_ECHO_PERFORMATIVE._serialized_start = 6508
    _ABCIMESSAGE_REQUEST_ECHO_PERFORMATIVE._serialized_end = 6552
    _ABCIMESSAGE_REQUEST_FLUSH_PERFORMATIVE._serialized_start = 6554
    _ABCIMESSAGE_REQUEST_FLUSH_PERFORMATIVE._serialized_end = 6582
    _ABCIMESSAGE_REQUEST_INFO_PERFORMATIVE._serialized_start = 6584
    _ABCIMESSAGE_REQUEST_INFO_PERFORMATIVE._serialized_end = 6672
    _ABCIMESSAGE_REQUEST_SET_OPTION_PERFORMATIVE._serialized_start = 6674
    _ABCIMESSAGE_REQUEST_SET_OPTION_PERFORMATIVE._serialized_end = 6749
    _ABCIMESSAGE_REQUEST_INIT_CHAIN_PERFORMATIVE._serialized_start = 6752
    _ABCIMESSAGE_REQUEST_INIT_CHAIN_PERFORMATIVE._serialized_end = 7099
    _ABCIMESSAGE_REQUEST_QUERY_PERFORMATIVE._serialized_start = 7101
    _ABCIMESSAGE_REQUEST_QUERY_PERFORMATIVE._serialized_end = 7194
    _ABCIMESSAGE_REQUEST_BEGIN_BLOCK_PERFORMATIVE._serialized_start = 7197
    _ABCIMESSAGE_REQUEST_BEGIN_BLOCK_PERFORMATIVE._serialized_end = 7460
    _ABCIMESSAGE_REQUEST_CHECK_TX_PERFORMATIVE._serialized_start = 7462
    _ABCIMESSAGE_REQUEST_CHECK_TX_PERFORMATIVE._serialized_end = 7568
    _ABCIMESSAGE_REQUEST_DELIVER_TX_PERFORMATIVE._serialized_start = 7570
    _ABCIMESSAGE_REQUEST_DELIVER_TX_PERFORMATIVE._serialized_end = 7615
    _ABCIMESSAGE_REQUEST_END_BLOCK_PERFORMATIVE._serialized_start = 7617
    _ABCIMESSAGE_REQUEST_END_BLOCK_PERFORMATIVE._serialized_end = 7665
    _ABCIMESSAGE_REQUEST_COMMIT_PERFORMATIVE._serialized_start = 7667
    _ABCIMESSAGE_REQUEST_COMMIT_PERFORMATIVE._serialized_end = 7696
    _ABCIMESSAGE_REQUEST_LIST_SNAPSHOTS_PERFORMATIVE._serialized_start = 7698
    _ABCIMESSAGE_REQUEST_LIST_SNAPSHOTS_PERFORMATIVE._serialized_end = 7735
    _ABCIMESSAGE_REQUEST_OFFER_SNAPSHOT_PERFORMATIVE._serialized_start = 7737
    _ABCIMESSAGE_REQUEST_OFFER_SNAPSHOT_PERFORMATIVE._serialized_end = 7856
    _ABCIMESSAGE_REQUEST_LOAD_SNAPSHOT_CHUNK_PERFORMATIVE._serialized_start = 7858
    _ABCIMESSAGE_REQUEST_LOAD_SNAPSHOT_CHUNK_PERFORMATIVE._serialized_end = 7953
    _ABCIMESSAGE_REQUEST_APPLY_SNAPSHOT_CHUNK_PERFORMATIVE._serialized_start = 7955
    _ABCIMESSAGE_REQUEST_APPLY_SNAPSHOT_CHUNK_PERFORMATIVE._serialized_end = 8050
    _ABCIMESSAGE_RESPONSE_EXCEPTION_PERFORMATIVE._serialized_start = 8052
    _ABCIMESSAGE_RESPONSE_EXCEPTION_PERFORMATIVE._serialized_end = 8100
    _ABCIMESSAGE_RESPONSE_ECHO_PERFORMATIVE._serialized_start = 8102
    _ABCIMESSAGE_RESPONSE_ECHO_PERFORMATIVE._serialized_end = 8147
    _ABCIMESSAGE_RESPONSE_FLUSH_PERFORMATIVE._serialized_start = 8149
    _ABCIMESSAGE_RESPONSE_FLUSH_PERFORMATIVE._serialized_end = 8178
    _ABCIMESSAGE_RESPONSE_INFO_PERFORMATIVE._serialized_start = 8181
    _ABCIMESSAGE_RESPONSE_INFO_PERFORMATIVE._serialized_end = 8322
    _ABCIMESSAGE_RESPONSE_SET_OPTION_PERFORMATIVE._serialized_start = 8324
    _ABCIMESSAGE_RESPONSE_SET_OPTION_PERFORMATIVE._serialized_end = 8399
    _ABCIMESSAGE_RESPONSE_INIT_CHAIN_PERFORMATIVE._serialized_start = 8402
    _ABCIMESSAGE_RESPONSE_INIT_CHAIN_PERFORMATIVE._serialized_end = 8640
    _ABCIMESSAGE_RESPONSE_QUERY_PERFORMATIVE._serialized_start = 8643
    _ABCIMESSAGE_RESPONSE_QUERY_PERFORMATIVE._serialized_end = 8856
    _ABCIMESSAGE_RESPONSE_BEGIN_BLOCK_PERFORMATIVE._serialized_start = 8858
    _ABCIMESSAGE_RESPONSE_BEGIN_BLOCK_PERFORMATIVE._serialized_end = 8953
    _ABCIMESSAGE_RESPONSE_CHECK_TX_PERFORMATIVE._serialized_start = 8956
    _ABCIMESSAGE_RESPONSE_CHECK_TX_PERFORMATIVE._serialized_end = 9160
    _ABCIMESSAGE_RESPONSE_DELIVER_TX_PERFORMATIVE._serialized_start = 9163
    _ABCIMESSAGE_RESPONSE_DELIVER_TX_PERFORMATIVE._serialized_end = 9369
    _ABCIMESSAGE_RESPONSE_END_BLOCK_PERFORMATIVE._serialized_start = 9372
    _ABCIMESSAGE_RESPONSE_END_BLOCK_PERFORMATIVE._serialized_end = 9672
    _ABCIMESSAGE_RESPONSE_COMMIT_PERFORMATIVE._serialized_start = 9674
    _ABCIMESSAGE_RESPONSE_COMMIT_PERFORMATIVE._serialized_end = 9741
    _ABCIMESSAGE_RESPONSE_LIST_SNAPSHOTS_PERFORMATIVE._serialized_start = 9743
    _ABCIMESSAGE_RESPONSE_LIST_SNAPSHOTS_PERFORMATIVE._serialized_end = 9847
    _ABCIMESSAGE_RESPONSE_OFFER_SNAPSHOT_PERFORMATIVE._serialized_start = 9849
    _ABCIMESSAGE_RESPONSE_OFFER_SNAPSHOT_PERFORMATIVE._serialized_end = 9947
    _ABCIMESSAGE_RESPONSE_LOAD_SNAPSHOT_CHUNK_PERFORMATIVE._serialized_start = 9949
    _ABCIMESSAGE_RESPONSE_LOAD_SNAPSHOT_CHUNK_PERFORMATIVE._serialized_end = 10007
    _ABCIMESSAGE_RESPONSE_APPLY_SNAPSHOT_CHUNK_PERFORMATIVE._serialized_start = 10010
    _ABCIMESSAGE_RESPONSE_APPLY_SNAPSHOT_CHUNK_PERFORMATIVE._serialized_end = 10162
    _ABCIMESSAGE_DUMMY_PERFORMATIVE._serialized_start = 10164
    _ABCIMESSAGE_DUMMY_PERFORMATIVE._serialized_end = 10269
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Usefule click utils."""

from typing import Callable

import click

from autonomy.analyse.abci.app_spec import DFA
from autonomy.deploy.image import ImageProfiles


def image_profile_flag(
    default: str = ImageProfiles.PRODUCTION, mark_default: bool = True
) -> Callable:
    """Choice of one flag between: '--local/--remote'."""

    def wrapper(f: Callable) -> Callable:
        for profile in ImageProfiles.ALL:
            f = click.option(
                f"--{profile}",
                "profile",
                flag_value=profile,
                help=f"To use the {profile} profile.",
                default=(profile == default) and mark_default,
            )(f)

        return f

    return wrapper


def abci_spec_format_flag(
    default: str = DFA.OutputFormats.YAML, mark_default: bool = True
) -> Callable:
    """Flags for abci spec outputs formats."""

    def wrapper(f: Callable) -> Callable:
        for of in DFA.OutputFormats.ALL:
            f = click.option(
                f"--{of}",
                "spec_format",
                flag_value=of,
                help=f"{of.title()} file.",
                default=(of == default) and mark_default,
            )(f)

        return f

    return wrapper
